@@ -1,8 +1,17 @@
+/*
+ * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
+ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
+#include <stdio.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <fcntl.h>
+
 #ifdef HAVE_LOCALE_H
 # include <locale.h>
 #endif
@@ -11,12 +20,17 @@
 # include <langinfo.h>
 #endif
 
-#include "Ecore.h"
-#include "ecore_private.h"
+#ifdef HAVE_SYS_MMAN_H
+# include <sys/mman.h>
+#endif
 
 #ifdef HAVE_EVIL
 # include <Evil.h>
 #endif
+#include <Eina.h>
+
+#include "Ecore.h"
+#include "ecore_private.h"
 
 static const char *_ecore_magic_string_get(Ecore_Magic m);
 static int _ecore_init_count = 0;
@@ -67,10 +81,15 @@ ecore_init(void)
 	     printf("WARNING: not a utf8 locale!\n");
 	  }
 	 */
+#ifdef HAVE_EVIL
+        evil_init();
+#endif
+	eina_init();
 	if (getenv("ECORE_FPS_DEBUG")) _ecore_fps_debug = 1;
 	if (_ecore_fps_debug) _ecore_fps_debug_init();
 	_ecore_signal_init();
 	_ecore_exe_init();
+	_ecore_loop_time = ecore_time_get();
      }
 
    return _ecore_init_count;
@@ -103,12 +122,16 @@ ecore_shutdown(void)
    _ecore_event_shutdown();
    _ecore_main_shutdown();
    _ecore_signal_shutdown();
+   eina_shutdown();
+#ifdef HAVE_EVIL
+   evil_shutdown();
+#endif
 
    return _ecore_init_count;
 }
 
 EAPI void
-_ecore_magic_fail(void *d, Ecore_Magic m, Ecore_Magic req_m, const char *fname)
+_ecore_magic_fail(const void *d, Ecore_Magic m, Ecore_Magic req_m, const char *fname)
 {
    fprintf(stderr,
 	   "\n"
