@@ -1,3 +1,14 @@
+/*
+ * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
+ */
+
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#include <stdio.h>
+#include <string.h>
+
 #include "ecore_private.h"
 #include "Ecore.h"
 #include "Ecore_Data.h"
@@ -23,10 +34,10 @@ static int _ecore_hash_node_add(Ecore_Hash *hash, Ecore_Hash_Node *node);
 static Ecore_Hash_Node * _ecore_hash_node_get(Ecore_Hash *hash, const void *key);
 static int _ecore_hash_increase(Ecore_Hash *hash);
 static int _ecore_hash_decrease(Ecore_Hash *hash);
-inline int _ecore_hash_rehash(Ecore_Hash *hash, Ecore_Hash_Node **old_table, int old_size);
+static inline int _ecore_hash_rehash(Ecore_Hash *hash, Ecore_Hash_Node **old_table, int old_size);
 static int _ecore_hash_bucket_destroy(Ecore_Hash_Node *list, Ecore_Free_Cb keyd,
 				      Ecore_Free_Cb valued);
-inline Ecore_Hash_Node * _ecore_hash_bucket_get(Ecore_Hash *hash,
+static inline Ecore_Hash_Node * _ecore_hash_bucket_get(Ecore_Hash *hash,
 						Ecore_Hash_Node *bucket, const void *key);
 
 static Ecore_Hash_Node *_ecore_hash_node_new(void *key, void *value);
@@ -218,7 +229,7 @@ ecore_hash_hash_set(Ecore_Hash *hash, Ecore_Hash *set)
  * @return  @c TRUE on success, @c FALSE on error.
  * @ingroup Ecore_Data_Hash_ADT_Destruction_Group
  */
-EAPI void 
+EAPI void
 ecore_hash_destroy(Ecore_Hash *hash)
 {
    unsigned int i = 0;
@@ -281,7 +292,7 @@ ecore_hash_count(Ecore_Hash *hash)
  * @return  TRUE on success, FALSE otherwise.
  * @ingroup Ecore_Data_Hash_ADT_Traverse_Group
  */
-EAPI int 
+EAPI int
 ecore_hash_for_each_node(Ecore_Hash *hash, Ecore_For_Each for_each_func, void *user_data)
 {
    unsigned int i = 0;
@@ -413,7 +424,7 @@ _ecore_hash_bucket_destroy(Ecore_Hash_Node *list, Ecore_Free_Cb keyd, Ecore_Free
 static int
 _ecore_hash_node_add(Ecore_Hash *hash, Ecore_Hash_Node *node)
 {
-   unsigned int hash_val;
+   unsigned long hash_val;
 
    CHECK_PARAM_POINTER_RETURN("hash", hash, FALSE);
    CHECK_PARAM_POINTER_RETURN("node", node, FALSE);
@@ -424,7 +435,7 @@ _ecore_hash_node_add(Ecore_Hash *hash, Ecore_Hash_Node *node)
 
    /* Compute the position in the table */
    if (!hash->hash_func)
-     hash_val = (unsigned int)node->key % ecore_prime_table[hash->size];
+     hash_val = (unsigned long)node->key % ecore_prime_table[hash->size];
    else
      hash_val = ECORE_COMPUTE_HASH(hash, node->key);
 
@@ -475,14 +486,14 @@ ecore_hash_remove(Ecore_Hash *hash, const void *key)
 {
    Ecore_Hash_Node *node = NULL;
    Ecore_Hash_Node *list;
-   unsigned int hash_val;
+   unsigned long hash_val;
    void *ret = NULL;
 
    CHECK_PARAM_POINTER_RETURN("hash", hash, NULL);
 
    /* Compute the position in the table */
    if (!hash->hash_func)
-     hash_val = (unsigned int )key % ecore_prime_table[hash->size];
+     hash_val = (unsigned long )key % ecore_prime_table[hash->size];
    else
      hash_val = ECORE_COMPUTE_HASH(hash, key);
 
@@ -580,7 +591,7 @@ ecore_hash_find(Ecore_Hash *hash, Ecore_Compare_Cb compare, const void *value)
 static Ecore_Hash_Node *
 _ecore_hash_node_get(Ecore_Hash *hash, const void *key)
 {
-   unsigned int hash_val;
+   unsigned long hash_val;
    Ecore_Hash_Node *node = NULL;
 
    CHECK_PARAM_POINTER_RETURN("hash", hash, NULL);
@@ -592,7 +603,7 @@ _ecore_hash_node_get(Ecore_Hash *hash, const void *key)
 
    /* Compute the position in the table */
    if (!hash->hash_func)
-     hash_val = (unsigned int )key % ecore_prime_table[hash->size];
+     hash_val = (unsigned long)key % ecore_prime_table[hash->size];
    else
      hash_val = ECORE_COMPUTE_HASH(hash, key);
 
@@ -621,7 +632,7 @@ _ecore_hash_node_get(Ecore_Hash *hash, const void *key)
  * @param key: the key to search for in the list
  * @return Returns NULL on error or not found, the found node on success
  */
-inline Ecore_Hash_Node *
+static inline Ecore_Hash_Node *
 _ecore_hash_bucket_get(Ecore_Hash *hash, Ecore_Hash_Node *bucket, const void *key)
 {
    Ecore_Hash_Node *prev = NULL;
@@ -773,7 +784,7 @@ _ecore_hash_decrease(Ecore_Hash *hash)
  * @param table: the table to remove the nodes from and place in hash
  * @return Returns TRUE on success, FALSE on error
  */
-inline int
+static inline int
 _ecore_hash_rehash(Ecore_Hash *hash, Ecore_Hash_Node **old_table, int old_size)
 {
    unsigned int i;
