@@ -1,3 +1,13 @@
+/*
+ * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
+ */
+
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#include <stdlib.h>
+
 #include "ecore_private.h"
 #include "Ecore.h"
 
@@ -13,8 +23,9 @@ static int          idlers_delete_me = 0;
  *
  * Add an idler handle to the event loop, returning a handle on success and
  * NULL otherwise.  The function @p func will be called repeatedly while
- * no other events are ready to be processed, as long as it returns 1. A return
- * of 0 deletes the idler.
+ * no other events are ready to be processed, as long as it returns 1
+ * (or ECORE_CALLBACK_RENEW). A return of 0 (or ECORE_CALLBACK_CANCEL) deletes
+ * the idler.
  *
  * Idlers are useful for progressively prossessing data without blocking.
  */
@@ -60,7 +71,7 @@ _ecore_idler_shutdown(void)
    while (idlers)
      {
 	Ecore_Idler *ie;
-	
+
 	ie = idlers;
 	idlers = _ecore_list2_remove(idlers, ie);
 	ECORE_MAGIC_SET(ie, ECORE_MAGIC_NONE);
@@ -72,24 +83,28 @@ _ecore_idler_shutdown(void)
 int
 _ecore_idler_call(void)
 {
-   Ecore_List2 *l;
+   Ecore_List2 *l, *last;
+
+   last = idlers ? ((Ecore_List2 *)idlers)->last : NULL;
 
    for (l = (Ecore_List2 *)idlers; l; l = l->next)
      {
 	Ecore_Idler *ie;
-	
+
 	ie = (Ecore_Idler *)l;
 	if (!ie->delete_me)
 	  {
 	     if (!ie->func(ie->data)) ecore_idler_del(ie);
 	  }
+
+	if (l == last) break;
      }
    if (idlers_delete_me)
      {
 	for (l = (Ecore_List2 *)idlers; l;)
 	  {
 	     Ecore_Idler *ie;
-	     
+
 	     ie = (Ecore_Idler *)l;
 	     l = l->next;
 	     if (ie->delete_me)
