@@ -8,13 +8,17 @@
 
 #ifdef _WIN32
 # include <winsock2.h>
+# define USER_TIMER_MINIMUM 0x0a
 #endif
 
+#include <stdlib.h>
+#include <stdio.h>
 #include <math.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <errno.h>
+#include <fcntl.h>
 
 #define FIX_HZ 1
 
@@ -25,11 +29,16 @@
 # endif
 #endif
 
+#ifdef HAVE_EVIL
+# include <Evil.h>
+#endif
+
 #include "ecore_private.h"
 #include "Ecore.h"
 
 static int  _ecore_main_select(double timeout);
 static void _ecore_main_fd_handlers_cleanup(void);
+static void _ecore_main_fd_handlers_bads_rem(void);
 static void _ecore_main_fd_handlers_call(void);
 static int  _ecore_main_fd_handlers_buf_call(void);
 static void _ecore_main_loop_iterate_internal(int once_only);
@@ -38,6 +47,7 @@ static int               in_main_loop = 0;
 static int               do_quit = 0;
 static Ecore_Fd_Handler *fd_handlers = NULL;
 static int               fd_handlers_delete_me = 0;
+static int (*main_loop_select)(int , fd_set *, fd_set *, fd_set *, struct timeval *) = select;
 
 static double            t1 = 0.0;
 static double            t2 = 0.0;
@@ -100,6 +110,37 @@ ecore_main_loop_quit(void)
    do_quit = 1;
 }
 
+/**
+ * Sets the function to use when monitoring multiple file descriptors,
+ * and waiting until one of more of the file descriptors before ready
+ * for some class of I/O operation.
+ *
+ * This function will be used instead of the system call select and
+ * could possible be used to integrate the Ecore event loop with an
+ * external event loop.
+ *
+ * @warning you don't know how to use, don't even try to use it.
+ *
+ * @ingroup Ecore_Main_Loop_Group
+ */
+EAPI void
+ecore_main_loop_select_func_set(int (*func)(int nfds, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout))
+{
+   main_loop_select = func;
+}
+
+/**
+ * Gets the select function set by ecore_select_func_set(),
+ * or the native select function if none was set.
+ *
+ * @ingroup Ecore_Main_Loop_Group
+ */
+EAPI void *
+ecore_main_loop_select_func_get(void)
+{
+   return main_loop_select;
+}
+
 /**
  * @defgroup Ecore_FD_Handler_Group File Event Handling Functions
  *
@@ -114,7 +155,7 @@ ecore_main_loop_quit(void)
  *
  * Normally the return value from the @p func is "zero means this handler is
  * finished and can be deleted" as is usual for handler callbacks.  However,
- * if the @p buf_func is supplied, then the return value from the @p func is 
+ * if the @p buf_func is supplied, then the return value from the @p func is
  * "non zero means the handler should be called again in a tight loop".
  *
  * @p buf_func is called during event loop handling to check if data that has
@@ -122,8 +163,8 @@ ecore_main_loop_quit(void)
  * read.  Some systems (notably xlib) handle their own buffering, and would
  * otherwise not work with select().  These systems should use a @p buf_func.
  * This is a most annoying hack, only ecore_x uses it, so refer to that for
- * an example.  NOTE - @p func should probably return "one" always if 
- * @p buf_func is used, to avoid confusion with the other return value 
+ * an example.  NOTE - @p func should probably return "one" always if
+ * @p buf_func is used, to avoid confusion with the other return value
  * semantics.
  *
  * @param   fd       The file descriptor to watch.
@@ -146,6 +187,7 @@ ecore_main_fd_handler_add(int fd, Ecore_Fd_Handler_Flags flags, int (*func) (voi
    if ((fd < 0) ||
        (flags == 0) ||
        (!func)) return NULL;
+
    fdh = calloc(1, sizeof(Ecore_Fd_Handler));
    if (!fdh) return NULL;
    ECORE_MAGIC_SET(fdh, ECORE_MAGIC_FD_HANDLER);
@@ -159,7 +201,8 @@ ecore_main_fd_handler_add(int fd, Ecore_Fd_Handler_Flags flags, int (*func) (voi
    fdh->data = (void *)data;
    fdh->buf_func = buf_func;
    fdh->buf_data = (void *)buf_data;
-   fd_handlers = _ecore_list2_append(fd_handlers, fdh);
+   fd_handlers = (Ecore_Fd_Handler *) eina_inlist_append(EINA_INLIST_GET(fd_handlers),
+							 EINA_INLIST_GET(fdh));
    return fdh;
 }
 
@@ -276,7 +319,8 @@ _ecore_main_shutdown(void)
 	Ecore_Fd_Handler *fdh;
 
 	fdh = fd_handlers;
-	fd_handlers = _ecore_list2_remove(fd_handlers, fdh);
+	fd_handlers = (Ecore_Fd_Handler *) eina_inlist_remove(EINA_INLIST_GET(fd_handlers),
+							      EINA_INLIST_GET(fdh));
 	ECORE_MAGIC_SET(fdh, ECORE_MAGIC_NONE);
 	free(fdh);
      }
@@ -290,7 +334,7 @@ _ecore_main_select(double timeout)
    fd_set         rfds, wfds, exfds;
    int            max_fd;
    int            ret;
-   Ecore_List2    *l;
+   Ecore_Fd_Handler *fdh;
 
    t = NULL;
    if ((!finite(timeout)) || (timeout == 0.0))  /* finite() tests for NaN, too big, too small, and infinity.  */
@@ -303,6 +347,11 @@ _ecore_main_select(double timeout)
      {
 	int sec, usec;
 
+#if _WIN32
+        if (timeout > 0.05)
+          timeout = 0.05;
+#endif
+
 #ifdef FIX_HZ
 	timeout += (0.5 / HZ);
 	sec = (int)timeout;
@@ -321,20 +370,11 @@ _ecore_main_select(double timeout)
    FD_ZERO(&exfds);
 
    /* call the prepare callback for all handlers */
-   for (l = (Ecore_List2 *)fd_handlers; l; l = l->next)
+   EINA_INLIST_FOREACH(fd_handlers, fdh)
+     if (!fdh->delete_me && fdh->prep_func)
+       fdh->prep_func (fdh->prep_data, fdh);
+   EINA_INLIST_FOREACH(fd_handlers, fdh)
      {
-	Ecore_Fd_Handler *fdh;
-
-	fdh = (Ecore_Fd_Handler *)l;
-
-	if (!fdh->delete_me && fdh->prep_func)
-		fdh->prep_func (fdh->prep_data, fdh);
-     }
-   for (l = (Ecore_List2 *)fd_handlers; l; l = l->next)
-     {
-	Ecore_Fd_Handler *fdh;
-
-	fdh = (Ecore_Fd_Handler *)l;
 	if (fdh->flags & ECORE_FD_READ)
 	  {
 	     FD_SET(fdh->fd, &rfds);
@@ -352,49 +392,89 @@ _ecore_main_select(double timeout)
 	  }
      }
    if (_ecore_signal_count_get()) return -1;
-   ret = select(max_fd + 1, &rfds, &wfds, &exfds, t);
+
+   ret = main_loop_select(max_fd + 1, &rfds, &wfds, &exfds, t);
+   _ecore_loop_time = ecore_time_get();
    if (ret < 0)
      {
 	if (errno == EINTR) return -1;
+	else if (errno == EBADF)
+	     _ecore_main_fd_handlers_bads_rem();
      }
    if (ret > 0)
      {
-	for (l = (Ecore_List2 *)fd_handlers; l; l = l->next)
-	  {
-	     Ecore_Fd_Handler *fdh;
-
-	     fdh = (Ecore_Fd_Handler *)l;
-	     if (!fdh->delete_me)
-	       {
-		  if (FD_ISSET(fdh->fd, &rfds))
-		    fdh->read_active = 1;
-		  if (FD_ISSET(fdh->fd, &wfds))
-		    fdh->write_active = 1;
-		  if (FD_ISSET(fdh->fd, &exfds))
-		    fdh->error_active = 1;
-	       }
-	  }
+	EINA_INLIST_FOREACH(fd_handlers, fdh)
+	  if (!fdh->delete_me)
+	    {
+	       if (FD_ISSET(fdh->fd, &rfds))
+		 fdh->read_active = 1;
+	       if (FD_ISSET(fdh->fd, &wfds))
+		 fdh->write_active = 1;
+	       if (FD_ISSET(fdh->fd, &exfds))
+		 fdh->error_active = 1;
+	    }
 	_ecore_main_fd_handlers_cleanup();
 	return 1;
      }
    return 0;
 }
 
+static void
+_ecore_main_fd_handlers_bads_rem(void)
+{
+   fprintf(stderr, "Removing bad fds\n");
+   Ecore_Fd_Handler *fdh;
+   Eina_Inlist *l;
+
+   for (l = EINA_INLIST_GET(fd_handlers); l; )
+     {
+	fdh = (Ecore_Fd_Handler *) l;
+	l = l->next;
+	errno = 0;
+
+	if ((fcntl(fdh->fd, F_GETFD) < 0) && (errno == EBADF))
+	  {
+	     fprintf(stderr, "Found bad fd at index %d\n", fdh->fd);
+	     if (fdh->flags & ECORE_FD_ERROR)
+	       {
+		  fprintf(stderr, "Fd set for error! calling user\n");
+	         if (!fdh->func(fdh->data, fdh))
+		   {
+		     fprintf(stderr, "Fd function err returned 0, remove it\n");
+		     fdh->delete_me = 1;
+		     fd_handlers_delete_me = 1;
+		     _ecore_main_fd_handlers_cleanup();
+		   }
+	       }
+	     else
+	       {
+		  fprintf(stderr, "Problematic fd found at %d! setting it for delete\n", fdh->fd);
+		  fdh->delete_me = 1;
+		  fd_handlers_delete_me = 1;
+		  _ecore_main_fd_handlers_cleanup();
+	       }
+	  }
+
+    }
+}
+
 static void
 _ecore_main_fd_handlers_cleanup(void)
 {
-   Ecore_List2 *l;
+   Ecore_Fd_Handler *fdh;
+   Eina_Inlist *l;
 
    if (!fd_handlers_delete_me) return;
-   for (l = (Ecore_List2 *)fd_handlers; l;)
+   for (l = EINA_INLIST_GET(fd_handlers); l; )
      {
-	Ecore_Fd_Handler *fdh;
+	fdh = (Ecore_Fd_Handler *) l;
 
-	fdh = (Ecore_Fd_Handler *)l;
 	l = l->next;
 	if (fdh->delete_me)
 	  {
-	     fd_handlers = _ecore_list2_remove(fd_handlers, fdh);
+//	     fprintf(stderr, "Removing fd %d\n", fdh->fd);
+	     fd_handlers = (Ecore_Fd_Handler *) eina_inlist_remove(EINA_INLIST_GET(fd_handlers),
+								   EINA_INLIST_GET(fdh));
 	     ECORE_MAGIC_SET(fdh, ECORE_MAGIC_NONE);
 	     free(fdh);
 	  }
@@ -405,145 +485,73 @@ _ecore_main_fd_handlers_cleanup(void)
 static void
 _ecore_main_fd_handlers_call(void)
 {
-   Ecore_List2    *l;
+   Ecore_Fd_Handler *fdh;
 
-   for (l = (Ecore_List2 *)fd_handlers; l; l = l->next)
-     {
-	Ecore_Fd_Handler *fdh;
-
-	fdh = (Ecore_Fd_Handler *)l;
-	if (!fdh->delete_me)
-	  {
-	     if ((fdh->read_active) ||
-		 (fdh->write_active) ||
-		 (fdh->error_active))
-	       {
-		  if (!fdh->func(fdh->data, fdh))
-		    {
-		       fdh->delete_me = 1;
-		       fd_handlers_delete_me = 1;
-		    }
-		  fdh->read_active = 0;
+   EINA_INLIST_FOREACH(fd_handlers, fdh)
+     if (!fdh->delete_me)
+       {
+	  if ((fdh->read_active) ||
+	      (fdh->write_active) ||
+	      (fdh->error_active))
+	    {
+	       if (!fdh->func(fdh->data, fdh))
+		 {
+		    fdh->delete_me = 1;
+		    fd_handlers_delete_me = 1;
+		 }
+	       fdh->read_active = 0;
 		  fdh->write_active = 0;
 		  fdh->error_active = 0;
-	       }
-	  }
-     }
+	    }
+       }
 }
 
 static int
 _ecore_main_fd_handlers_buf_call(void)
 {
-   Ecore_List2    *l;
+   Ecore_Fd_Handler *fdh;
    int ret;
 
    ret = 0;
-   for (l = (Ecore_List2 *)fd_handlers; l; l = l->next)
-     {
-	Ecore_Fd_Handler *fdh;
-
-	fdh = (Ecore_Fd_Handler *)l;
-	if (!fdh->delete_me)
-	  {
-	     if (fdh->buf_func)
-	       {
-		  if (fdh->buf_func(fdh->buf_data, fdh))
-		    {
-		       ret |= fdh->func(fdh->data, fdh);
-		       fdh->read_active = 1;
-		    }
-	       }
-	  }
-     }
-   return ret;
-}
-
-static int
-_ecore_main_win32_message(double timeout)
-{
-#ifdef _WIN32
-   MSG msg;
-
-   if (!finite(timeout))
-     timeout = 0.0;
-
-   if (timeout < 0.0)
-     {
-       for (;;)
-         {
-            if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
-              return 1;
-         }
-     }
-   else
-     {
-        double start;
-        double t;
-
-        start = ecore_time_get();
-        while ((t = ecore_time_get()) < (start + timeout))
-          {
-             if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
-               return 1;
-          }
-     }
-#endif /* _WIN32 */
-
-   return 0;
-}
-
-static int
-_ecore_main_win32_message_buf_call(void)
-{
-#ifdef _WIN32
-   MSG msg;
-   int ret;
-
-   ret = 0;
-   if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
-     {
-        if ((msg.message & 0xffff) < 0xC000)
-          {
-             TranslateMessage(&msg);
-             DispatchMessage(&msg);
-          }
-        ret = 1;
-     }
+   EINA_INLIST_FOREACH(fd_handlers, fdh)
+     if (!fdh->delete_me)
+       {
+	  if (fdh->buf_func)
+	    {
+	       if (fdh->buf_func(fdh->buf_data, fdh))
+		 {
+		    ret |= fdh->func(fdh->data, fdh);
+		    fdh->read_active = 1;
+		 }
+	    }
+       }
 
    return ret;
-#else
-   return 0;
-#endif /* _WIN32 */
 }
 
 static void
 _ecore_main_loop_iterate_internal(int once_only)
 {
-   double next_time;
+   double next_time = -1.0;
    int    have_event = 0;
    int    have_signal;
-   int    have_msg = 0;
 
    in_main_loop++;
    /* expire any timers */
      {
 	double now;
 
-	now = ecore_time_get();
-	while (_ecore_timer_call(now));
+	now = ecore_loop_time_get();
+        while (_ecore_timer_call(now));
 	_ecore_timer_cleanup();
      }
-   /* any timers re-added as a result of these are allowed to go */
-   _ecore_timer_enable_new();
    /* process signals into events .... */
    while (_ecore_signal_count_get()) _ecore_signal_call();
    if (_ecore_event_exist())
      {
 	have_event = 1;
 	have_signal = 1;
-	have_msg = 1;
 	_ecore_main_select(0.0);
-	_ecore_main_win32_message(0.0);
 	goto process_events;
      }
    /* call idle enterers ... */
@@ -551,29 +559,25 @@ _ecore_main_loop_iterate_internal(int once_only)
      _ecore_idle_enterer_call();
    else
      {
-	have_event = have_signal = have_msg = 0;
+	have_event = have_signal = 0;
 
 	if (_ecore_main_select(0.0) > 0) have_event = 1;
 
-	if (_ecore_main_win32_message(0.0) > 0) have_msg = 1;
 	if (_ecore_signal_count_get() > 0) have_signal = 1;
 
-	if (have_signal || have_event || have_msg)
+	if (have_signal || have_event)
 	  goto process_events;
      }
 
    /* if these calls caused any buffered events to appear - deal with them */
    while (_ecore_main_fd_handlers_buf_call());
-   while (_ecore_main_win32_message_buf_call());
 
    /* if ther are any - jump to processing them */
    if (_ecore_event_exist())
      {
 	have_event = 1;
 	have_signal = 1;
-	have_msg = 1;
 	_ecore_main_select(0.0);
-	_ecore_main_win32_message(0.0);
 	goto process_events;
      }
    if (once_only)
@@ -590,6 +594,8 @@ _ecore_main_loop_iterate_internal(int once_only)
 	  _ecore_fps_debug_runtime_add(t2 - t1);
      }
    start_loop:
+   /* any timers re-added as a result of these are allowed to go */
+   _ecore_timer_enable_new();
    if (do_quit)
      {
 	in_main_loop--;
@@ -598,7 +604,7 @@ _ecore_main_loop_iterate_internal(int once_only)
    if (!_ecore_event_exist())
      {
 	/* init flags */
-	have_event = have_signal = have_msg = 0;
+	have_event = have_signal = 0;
 	next_time = _ecore_timer_next_get();
 	/* no timers */
 	if (next_time < 0)
@@ -607,7 +613,6 @@ _ecore_main_loop_iterate_internal(int once_only)
 	     if (!_ecore_idler_exist())
 	       {
 		  if (_ecore_main_select(-1.0) > 0) have_event = 1;
-		  if (_ecore_main_win32_message(-1.0) > 0) have_msg = 1;
 		  if (_ecore_signal_count_get() > 0) have_signal = 1;
 	       }
 	     /* idlers */
@@ -618,13 +623,13 @@ _ecore_main_loop_iterate_internal(int once_only)
 		       if (!_ecore_idler_call()) goto start_loop;
 		       if (_ecore_event_exist()) break;
 		       if (_ecore_main_select(0.0) > 0) have_event = 1;
-		       if (_ecore_main_win32_message(0.0) > 0) have_msg = 1;
 		       if (_ecore_signal_count_get() > 0) have_signal = 1;
-		       if (have_event || have_signal || have_msg) break;
+		       if (have_event || have_signal) break;
 		       next_time = _ecore_timer_next_get();
 		       if (next_time >= 0) goto start_loop;
 		       if (do_quit) break;
 		    }
+                  _ecore_loop_time = ecore_time_get();
 	       }
 	  }
 	/* timers */
@@ -634,7 +639,6 @@ _ecore_main_loop_iterate_internal(int once_only)
 	     if (!_ecore_idler_exist())
 	       {
 		  if (_ecore_main_select(next_time) > 0) have_event = 1;
-		  if (_ecore_main_win32_message(next_time) > 0) have_msg = 1;
 		  if (_ecore_signal_count_get() > 0) have_signal = 1;
 	       }
 	     /* idlers */
@@ -642,21 +646,16 @@ _ecore_main_loop_iterate_internal(int once_only)
 	       {
 		  for (;;)
 		    {
-		       double cur_time, t;
-
 		       if (!_ecore_idler_call()) goto start_loop;
 		       if (_ecore_event_exist()) break;
 		       if (_ecore_main_select(0.0) > 0) have_event = 1;
-		       if (_ecore_main_win32_message(0.0) > 0) have_msg = 1;
 		       if (_ecore_signal_count_get() > 0) have_signal = 1;
-		       if (have_event || have_signal || have_msg) break;
-		       cur_time = ecore_time_get();
-		       t = ecore_time_get() - cur_time;
-		       if (t >= next_time) break;
+		       if (have_event || have_signal) break;
 		       next_time = _ecore_timer_next_get();
 		       if (next_time < 0) goto start_loop;
 		       if (do_quit) break;
 		    }
+                  _ecore_loop_time = ecore_time_get();
 	       }
 	  }
      }
@@ -681,8 +680,41 @@ _ecore_main_loop_iterate_internal(int once_only)
 	_ecore_main_fd_handlers_cleanup();
      }
    while (_ecore_main_fd_handlers_buf_call());
-   if (have_msg) _ecore_main_win32_message_buf_call();
-   while (_ecore_main_win32_message_buf_call());
+#if _WIN32
+   {
+      MSG msg;
+      BOOL ret;
+      UINT_PTR TmrID = 0;
+      if ((next_time > 0) && ((UINT) (next_time * 1000.0) > USER_TIMER_MINIMUM))
+
+        {
+           TmrID = SetTimer(NULL, 0, (UINT) (next_time * 1000.0), NULL);
+           ret = GetMessage(&msg, NULL, 0, 0);
+        }
+      else
+        {
+           ret = PeekMessage(&msg, NULL, 0, 0, PM_REMOVE);
+        }
+
+      if (ret)
+        {
+           do
+             {
+                TranslateMessage(&msg);
+                DispatchMessageW(&msg);
+                Sleep(0); /* Give other threads a chance to run */
+             } while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE));
+        }
+
+      if (TmrID)
+        {
+           KillTimer(NULL, TmrID);
+           TmrID = 0;
+        }
+   }
+#endif
+
+
 /* ok - too much optimising. let's call idle enterers more often. if we
  * have events that place more events or jobs etc. on the event queue
  * we may never get to call an idle enterer
