@@ -20,43 +20,70 @@
  * bear in mind that each one can only perform one operation at a time.
  * You need to wait for the ECORE_CON_EVENT_URL_COMPLETE event before re-using
  * or destroying the object.
- * 
+ *
  * Example Usage 1 (HTTP GET):
  *   ecore_con_url_url_set(url_con, "http://www.google.com");
- *   ecore_con_url_send(url, NULL, 0, NULL);
+ *   ecore_con_url_send(url_con, NULL, 0, NULL);
  *
  * Example usage 2 (HTTP POST):
  *   ecore_con_url_url_set(url_con, "http://www.example.com/post_handler.cgi");
- *   ecore_con_url_send(url, data, data_length, "multipart/form-data");
+ *   ecore_con_url_send(url_con, data, data_length, "multipart/form-data");
  *
  * Example Usage 3 (FTP download):
  *   ecore_con_url_url_set(url_con, "ftp://ftp.example.com/pub/myfile");
- *   ecore_con_url_send(url, NULL, 0, NULL);
+ *   ecore_con_url_send(url_con, NULL, 0, NULL);
  *
- * FIXME: Support more CURL features: Authentication, FTP upload, Progress callbacks and more...
+ * Example Usage 4 (FTP upload as ftp://ftp.example.com/file):
+ *   ecore_con_url_url_set(url_con, "ftp://ftp.example.com");
+ *   ecore_con_url_ftp_upload(url_con, "/tmp/file", "user", "pass", NULL);
+ *
+ * Example Usage 5 (FTP upload as ftp://ftp.example.com/dir/file):
+ *   ecore_con_url_url_set(url_con, "ftp://ftp.example.com");
+ *   ecore_con_url_ftp_upload(url_con, "/tmp/file", "user", "pass","dir");
+ *
+ * FIXME: Support more CURL features: Authentication, Progress callbacks and more...
  */
+
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#include <string.h>
+#include <errno.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+
 #include "Ecore.h"
 #include "ecore_private.h"
 #include "Ecore_Con.h"
 #include "ecore_con_private.h"
 
-#ifdef HAVE_CURL
-static int _ecore_con_url_fd_handler(void *data, Ecore_Fd_Handler *fd_handler);
-static int _ecore_con_url_perform(Ecore_Con_Url *url_con);
-static size_t _ecore_con_url_data_cb(void *buffer, size_t size, size_t nmemb, void *userp);
-static int _ecore_con_url_progress_cb(void *clientp, double dltotal, double dlnow, double ultotal, double ulnow);
-static void _ecore_con_event_url_free(void *data __UNUSED__, void *ev);
-static int _ecore_con_url_process_completed_jobs(Ecore_Con_Url *url_con_to_match);
+/**
+ * @defgroup Ecore_Con_Url_Group Ecore URL Connection Functions
+ *
+ * Utility functions that set up, use and shut down the Ecore URL
+ * Connection library.
+ * FIXME: write detailed description
+ */
 
 int ECORE_CON_EVENT_URL_DATA = 0;
 int ECORE_CON_EVENT_URL_COMPLETE = 0;
-int ECORE_CON_EVENT_URL_PROGRESS_DOWNLOAD = 0;
-int ECORE_CON_EVENT_URL_PROGRESS_UPLOAD = 0;
+int ECORE_CON_EVENT_URL_PROGRESS = 0;
 
-static CURLM *curlm = NULL;
-static Ecore_List *_url_con_list = NULL;
-static fd_set _current_fd_set;
-static int init_count = 0;
+#ifdef HAVE_CURL
+static int _ecore_con_url_fd_handler(void *data, Ecore_Fd_Handler *fd_handler);
+static int _ecore_con_url_perform(Ecore_Con_Url *url_con);
+static size_t _ecore_con_url_data_cb(void *buffer, size_t size, size_t nitems, void *userp);
+static int _ecore_con_url_progress_cb(void *clientp, double dltotal, double dlnow, double ultotal, double ulnow);
+static size_t _ecore_con_url_read_cb(void *ptr, size_t size, size_t nitems, void *stream);
+static void _ecore_con_event_url_free(void *data __UNUSED__, void *ev);
+static int _ecore_con_url_process_completed_jobs(Ecore_Con_Url *url_con_to_match);
+
+static Ecore_Idler	*_fd_idler_handler = NULL;
+static Eina_List	*_url_con_list = NULL;
+static CURLM		*curlm = NULL;
+static fd_set		 _current_fd_set;
+static int		 init_count = 0;
 
 struct _Ecore_Con_Url_Event
 {
@@ -92,6 +119,12 @@ _url_complete_push_event(int type, void *ev)
 
 #endif
 
+/**
+ * Initialises the Ecore_Con_Url library.
+ * @return Number of times the library has been initialised without being
+ *          shut down.
+ * @ingroup Ecore_Con_Url_Group
+ */
 EAPI int
 ecore_con_url_init(void)
 {
@@ -100,14 +133,7 @@ ecore_con_url_init(void)
      {
 	ECORE_CON_EVENT_URL_DATA = ecore_event_type_new();
 	ECORE_CON_EVENT_URL_COMPLETE = ecore_event_type_new();
-	ECORE_CON_EVENT_URL_PROGRESS_DOWNLOAD = ecore_event_type_new();
-	ECORE_CON_EVENT_URL_PROGRESS_UPLOAD = ecore_event_type_new();
-     }
-
-   if (!_url_con_list)
-     {
-	_url_con_list = ecore_list_new();
-	if (!_url_con_list) return 0;
+	ECORE_CON_EVENT_URL_PROGRESS = ecore_event_type_new();
      }
 
    if (!curlm)
@@ -115,16 +141,16 @@ ecore_con_url_init(void)
 	FD_ZERO(&_current_fd_set);
 	if (curl_global_init(CURL_GLOBAL_NOTHING))
 	  {
-	     ecore_list_destroy(_url_con_list);
-	     _url_con_list = NULL;
+	     while (_url_con_list)
+	       ecore_con_url_destroy(eina_list_data_get(_url_con_list));
 	     return 0;
 	  }
 
 	curlm = curl_multi_init();
 	if (!curlm)
 	  {
-	     ecore_list_destroy(_url_con_list);
-	     _url_con_list = NULL;
+	     while (_url_con_list)
+	       ecore_con_url_destroy(eina_list_data_get(_url_con_list));
 	     return 0;
 	  }
      }
@@ -135,28 +161,21 @@ ecore_con_url_init(void)
 #endif
 }
 
+/**
+ * Shuts down the Ecore_Con_Url library.
+ * @return  Number of calls that still uses Ecore_Con_Url
+ * @ingroup Ecore_Con_Url_Group
+ */
 EAPI int
 ecore_con_url_shutdown(void)
 {
 #ifdef HAVE_CURL
-
    if (!init_count)
      return 0;
 
    init_count--;
-   if (_url_con_list)
-     {
-	if (!ecore_list_empty_is(_url_con_list))
-	  {
-	     Ecore_Con_Url *url_con;
-	     while ((url_con = ecore_list_first_remove(_url_con_list)))
-	       {
-		  ecore_con_url_destroy(url_con);
-	       }
-	  }
-	ecore_list_destroy(_url_con_list);
-	_url_con_list = NULL;
-     }
+   while (_url_con_list)
+     ecore_con_url_destroy(eina_list_data_get(_url_con_list));
 
    if (curlm)
      {
@@ -169,6 +188,11 @@ ecore_con_url_shutdown(void)
    return 1;
 }
 
+/**
+ * Creates and initializes a new Ecore_Con_Url.
+ * @return  NULL on error, a new Ecore_Con_Url on success.
+ * @ingroup Ecore_Con_Url_Group
+ */
 EAPI Ecore_Con_Url *
 ecore_con_url_new(const char *url)
 {
@@ -206,6 +230,11 @@ ecore_con_url_new(const char *url)
    curl_easy_setopt(url_con->curl_easy, CURLOPT_TIMEOUT, 300);
    curl_easy_setopt(url_con->curl_easy, CURLOPT_FOLLOWLOCATION, 1);
 
+   curl_easy_setopt(url_con->curl_easy, CURLOPT_ENCODING, "gzip,deflate");
+
+   url_con->fd = -1;
+   url_con->write_fd = -1;
+
    return url_con;
 #else
    return NULL;
@@ -213,6 +242,11 @@ ecore_con_url_new(const char *url)
 #endif
 }
 
+/**
+ * Frees the Ecore_Con_Url.
+ * @return  FIXME: To be documented.
+ * @ingroup Ecore_Con_Url_Group
+ */
 EAPI void
 ecore_con_url_destroy(Ecore_Con_Url *url_con)
 {
@@ -226,13 +260,21 @@ ecore_con_url_destroy(Ecore_Con_Url *url_con)
 
    ECORE_MAGIC_SET(url_con, ECORE_MAGIC_NONE);
    if (url_con->fd_handler)
-     ecore_main_fd_handler_del(url_con->fd_handler);
+     {
+	ecore_main_fd_handler_del(url_con->fd_handler);
+	url_con->fd = -1;
+     }
    if (url_con->curl_easy)
      {
 	if (url_con->active)
-	  curl_multi_remove_handle(curlm, url_con->curl_easy);
+	  {
+	     url_con->active = 0;
+
+	     curl_multi_remove_handle(curlm, url_con->curl_easy);
+	  }
 	curl_easy_cleanup(url_con->curl_easy);
      }
+   _url_con_list = eina_list_remove(_url_con_list, url_con);
    curl_slist_free_all(url_con->headers);
    free(url_con->url);
    free(url_con);
@@ -242,6 +284,11 @@ ecore_con_url_destroy(Ecore_Con_Url *url_con)
 #endif
 }
 
+/**
+ * FIXME: To be documented.
+ * @return  FIXME: To be documented.
+ * @ingroup Ecore_Con_Url_Group
+ */
 EAPI int
 ecore_con_url_url_set(Ecore_Con_Url *url_con, const char *url)
 {
@@ -267,6 +314,11 @@ ecore_con_url_url_set(Ecore_Con_Url *url_con, const char *url)
 #endif
 }
 
+/**
+ * FIXME: To be documented.
+ * @return  FIXME: To be documented.
+ * @ingroup Ecore_Con_Url_Group
+ */
 EAPI void
 ecore_con_url_data_set(Ecore_Con_Url *url_con, void *data)
 {
@@ -285,6 +337,11 @@ ecore_con_url_data_set(Ecore_Con_Url *url_con, void *data)
 #endif
 }
 
+/**
+ * FIXME: To be documented.
+ * @return  FIXME: To be documented.
+ * @ingroup Ecore_Con_Url_Group
+ */
 EAPI void *
 ecore_con_url_data_get(Ecore_Con_Url *url_con)
 {
@@ -302,6 +359,11 @@ ecore_con_url_data_get(Ecore_Con_Url *url_con)
 #endif
 }
 
+/**
+ * FIXME: To be documented.
+ * @return  FIXME: To be documented.
+ * @ingroup Ecore_Con_Url_Group
+ */
 EAPI void
 ecore_con_url_time(Ecore_Con_Url *url_con, Ecore_Con_Url_Time condition, time_t tm)
 {
@@ -322,8 +384,51 @@ ecore_con_url_time(Ecore_Con_Url *url_con, Ecore_Con_Url_Time condition, time_t
 #endif
 }
 
+/**
+ * FIXME: To be documented.
+ * @return  FIXME: To be documented.
+ * @ingroup Ecore_Con_Url_Group
+ */
+EAPI void
+ecore_con_url_fd_set(Ecore_Con_Url *url_con, int fd)
+{
+#ifdef HAVE_CURL
+   if (!ECORE_MAGIC_CHECK(url_con, ECORE_MAGIC_CON_URL))
+     {
+	ECORE_MAGIC_FAIL(url_con, ECORE_MAGIC_CON_URL, "ecore_con_url_set");
+	return ;
+     }
+   url_con->write_fd = fd;
+#endif
+}
+
+/**
+ * FIXME: To be documented.
+ * @return  FIXME: To be documented.
+ * @ingroup Ecore_Con_Url_Group
+ */
 EAPI int
-ecore_con_url_send(Ecore_Con_Url *url_con, void *data, size_t length, char *content_type)
+ecore_con_url_received_bytes_get(Ecore_Con_Url *url_con)
+{
+#ifdef HAVE_CURL
+   if (!ECORE_MAGIC_CHECK(url_con, ECORE_MAGIC_CON_URL))
+     {
+	ECORE_MAGIC_FAIL(url_con, ECORE_MAGIC_CON_URL, "ecore_con_url_received_bytes_get");
+	return -1;
+     }
+
+   return url_con->received;
+#endif
+   return 0;
+}
+
+/**
+ * FIXME: To be documented.
+ * @return  FIXME: To be documented.
+ * @ingroup Ecore_Con_Url_Group
+ */
+EAPI int
+ecore_con_url_send(Ecore_Con_Url *url_con, const void *data, size_t length, const char *content_type)
 {
 #ifdef HAVE_CURL
    char tmp[256];
@@ -350,7 +455,7 @@ ecore_con_url_send(Ecore_Con_Url *url_con, void *data, size_t length, char *cont
 	     sprintf(tmp, "Content-type: %s", content_type);
 	     url_con->headers = curl_slist_append(url_con->headers, tmp);
 	  }
-	sprintf(tmp, "Content-length: %d", length);
+	sprintf(tmp, "Content-length: %zu", length);
 	url_con->headers = curl_slist_append(url_con->headers, tmp);
      }
 
@@ -375,6 +480,8 @@ ecore_con_url_send(Ecore_Con_Url *url_con, void *data, size_t length, char *cont
 
    curl_easy_setopt(url_con->curl_easy, CURLOPT_HTTPHEADER, url_con->headers);
 
+   url_con->received = 0;
+
    return _ecore_con_url_perform(url_con);
 #else
    return 0;
@@ -385,24 +492,207 @@ ecore_con_url_send(Ecore_Con_Url *url_con, void *data, size_t length, char *cont
 #endif
 }
 
+/**
+ * Makes a FTP upload
+ * @return  FIXME: To be more documented.
+ * @ingroup Ecore_Con_Url_Group
+ */
+EAPI int
+ecore_con_url_ftp_upload(Ecore_Con_Url *url_con, const char *filename, const char *user, const char *pass, const char *upload_dir)
+{
 #ifdef HAVE_CURL
+   char url[4096];
+   char userpwd[4096];
+   FILE *fd;
+   struct stat file_info;
+
+   if (!ECORE_MAGIC_CHECK(url_con, ECORE_MAGIC_CON_URL))
+     {
+	ECORE_MAGIC_FAIL(url_con, ECORE_MAGIC_CON_URL, "ecore_con_url_ftp_upload");
+	return 0;
+     }
+
+   if (url_con->active) return 0;
+   if (!url_con->url) return 0;
+   if (filename)
+     {
+	if (stat(filename, &file_info)) return 0;
+	fd = fopen(filename, "rb");
+	if (upload_dir)
+	   snprintf(url, sizeof(url), "ftp://%s/%s/%s", url_con->url, upload_dir, basename(filename));
+	else
+	   snprintf(url, sizeof(url), "ftp://%s/%s", url_con->url, basename(filename));
+	snprintf(userpwd, sizeof(userpwd), "%s:%s", user, pass);
+	curl_easy_setopt(url_con->curl_easy, CURLOPT_INFILESIZE_LARGE, (curl_off_t)file_info.st_size);
+	curl_easy_setopt(url_con->curl_easy, CURLOPT_USERPWD, userpwd);
+	curl_easy_setopt(url_con->curl_easy, CURLOPT_UPLOAD, 1);
+	curl_easy_setopt(url_con->curl_easy, CURLOPT_READFUNCTION, _ecore_con_url_read_cb);
+	curl_easy_setopt(url_con->curl_easy, CURLOPT_READDATA, fd);
+	ecore_con_url_url_set(url_con, url);
+
+	return _ecore_con_url_perform(url_con);
+     }
+   else
+	return 0;
+#else
+   return 0;
+   url_con = NULL;
+   filename = NULL;
+   user = NULL;
+   pass = NULL;
+   upload_dir = NULL;
+#endif
+}
+
+/**
+ * Enable or disable libcurl verbose output, useful for debug
+ * @return  FIXME: To be more documented.
+ * @ingroup Ecore_Con_Url_Group
+ */
+EAPI void
+ecore_con_url_verbose_set(Ecore_Con_Url *url_con, int verbose)
+{
+#ifdef HAVE_CURL
+   if (!ECORE_MAGIC_CHECK(url_con, ECORE_MAGIC_CON_URL))
+     {
+	ECORE_MAGIC_FAIL(url_con, ECORE_MAGIC_CON_URL, "ecore_con_url_verbose_set");
+	return;
+     }
+
+   if (url_con->active) return;
+   if (!url_con->url) return;
+   if (verbose == TRUE)
+	curl_easy_setopt(url_con->curl_easy, CURLOPT_VERBOSE, 1);
+   else
+	curl_easy_setopt(url_con->curl_easy, CURLOPT_VERBOSE, 0);
+#endif
+}
+
+/**
+ * Enable or disable EPSV extension
+ * @return  FIXME: To be more documented.
+ * @ingroup Ecore_Con_Url_Group
+ */
+EAPI void
+ecore_con_url_ftp_use_epsv_set(Ecore_Con_Url *url_con, int use_epsv)
+{
+#ifdef HAVE_CURL
+   if (!ECORE_MAGIC_CHECK(url_con, ECORE_MAGIC_CON_URL))
+     {
+	ECORE_MAGIC_FAIL(url_con, ECORE_MAGIC_CON_URL, "ecore_con_url_ftp_use_epsv_set");
+	return;
+     }
+
+   if (url_con->active) return;
+   if (!url_con->url) return;
+   if (use_epsv == TRUE)
+	curl_easy_setopt(url_con->curl_easy, CURLOPT_FTP_USE_EPSV, 1);
+   else
+	curl_easy_setopt(url_con->curl_easy, CURLOPT_FTP_USE_EPSV, 0);
+#endif
+}
+
+#ifdef HAVE_CURL
+static int
+_ecore_con_url_suspend_fd_handler(void)
+{
+   Eina_List		*l;
+   Ecore_Con_Url	*url_con;
+   int			 deleted = 0;
+
+   if (!_url_con_list)
+     return 0;
+
+   EINA_LIST_FOREACH(_url_con_list, l, url_con)
+     {
+	if (url_con->active && url_con->fd_handler)
+	  {
+	     ecore_main_fd_handler_del(url_con->fd_handler);
+	     url_con->fd_handler = NULL;
+	     deleted++;
+	  }
+     }
+
+   return deleted;
+}
+
+static int
+_ecore_con_url_restart_fd_handler(void)
+{
+   Eina_List		*l;
+   Ecore_Con_Url	*url_con;
+   int			 activated = 0;
+
+   if (!_url_con_list)
+     return 0;
+
+   EINA_LIST_FOREACH(_url_con_list, l, url_con)
+     {
+	if (url_con->fd_handler == NULL && url_con->fd != -1)
+	  {
+	     url_con->fd_handler = ecore_main_fd_handler_add(url_con->fd,
+							     url_con->flags,
+							     _ecore_con_url_fd_handler,
+							     NULL, NULL, NULL);
+	     activated++;
+	  }
+     }
+
+   return activated;
+}
+
 static size_t
-_ecore_con_url_data_cb(void *buffer, size_t size, size_t nmemb, void *userp)
+_ecore_con_url_data_cb(void *buffer, size_t size, size_t nitems, void *userp)
 {
    Ecore_Con_Url *url_con;
    Ecore_Con_Event_Url_Data *e;
-   size_t real_size = size * nmemb;
+   size_t real_size = size * nitems;
 
    url_con = (Ecore_Con_Url *)userp;
-   e = malloc(sizeof(Ecore_Con_Event_Url_Data) + sizeof(unsigned char) * (real_size - 1));
-   if (e)
+
+   if (!url_con) return -1;
+   if (!ECORE_MAGIC_CHECK(url_con, ECORE_MAGIC_CON_URL))
      {
-	e->url_con = url_con;
-	e->size = real_size;
-	memcpy(e->data, buffer, real_size);
-	ecore_event_add(ECORE_CON_EVENT_URL_DATA, e,
-			_ecore_con_event_url_free, NULL);
+	ECORE_MAGIC_FAIL(url_con, ECORE_MAGIC_CON_URL, "ecore_con_url_data_cb");
+	return -1;
      }
+
+   url_con->received += real_size;
+
+   if (url_con->write_fd < 0)
+     {
+	e = malloc(sizeof(Ecore_Con_Event_Url_Data) + sizeof(unsigned char) * (real_size - 1));
+	if (e)
+	  {
+	     e->url_con = url_con;
+	     e->size = real_size;
+	     memcpy(e->data, buffer, real_size);
+	     ecore_event_add(ECORE_CON_EVENT_URL_DATA, e,
+			     _ecore_con_event_url_free, NULL);
+	  }
+     }
+   else
+     {
+	ssize_t	count = 0;
+	size_t	total_size = real_size;
+	size_t	offset = 0;
+
+	while (total_size > 0)
+	  {
+	     count = write(url_con->write_fd, (char*) buffer + offset, total_size);
+	     if (count < 0)
+	       {
+		  if (errno != EAGAIN && errno != EINTR)
+		    return -1;
+	       }
+	     else
+	       {
+		  total_size -= count;
+		  offset += count;
+	       }
+	  }
+     }
+
    return real_size;
 }
 
@@ -425,16 +715,40 @@ _ecore_con_url_data_cb(void *buffer, size_t size, size_t nmemb, void *userp)
 static int
 _ecore_con_url_progress_cb(void *clientp, double dltotal, double dlnow, double ultotal, double ulnow)
 {
-   Ecore_Con_Url *url_con;
+   Ecore_Con_Event_Url_Progress	*e;
+   Ecore_Con_Url		*url_con;
 
    url_con = clientp;
 
-   ECORE_CON_URL_TRANSMISSION(Download, ECORE_CON_EVENT_URL_PROGRESS_DOWNLOAD, url_con, dltotal, dlnow);
-   ECORE_CON_URL_TRANSMISSION(Upload, ECORE_CON_EVENT_URL_PROGRESS_UPLOAD, url_con, ultotal, ulnow);
+   e = malloc(sizeof(Ecore_Con_Event_Url_Progress));
+   if (e)
+     {
+	e->url_con = url_con;
+	e->down.total = dltotal;
+	e->down.now = dlnow;
+	e->up.total = ultotal;
+	e->up.now = ulnow;
+	ecore_event_add(ECORE_CON_EVENT_URL_PROGRESS, e, _ecore_con_event_url_free, NULL);
+     }
 
    return 0;
 }
 
+static size_t
+_ecore_con_url_read_cb(void *ptr, size_t size, size_t nitems, void *stream)
+{
+   size_t retcode = fread(ptr, size, nitems, stream);
+   if (ferror((FILE*)stream)) {
+	fclose(stream);
+	return CURL_READFUNC_ABORT;
+   } else if ((retcode == 0) || (retcode < nitems)) {
+	fclose((FILE*)stream);
+	return 0;
+   }
+   fprintf(stderr, "*** We read %zu bytes from file\n", retcode);
+   return retcode;
+}
+
 static int
 _ecore_con_url_perform(Ecore_Con_Url *url_con)
 {
@@ -445,10 +759,11 @@ _ecore_con_url_perform(Ecore_Con_Url *url_con)
    int still_running;
    int completed_immediately = 0;
 
-   ecore_list_append(_url_con_list, url_con);
+   _url_con_list = eina_list_append(_url_con_list, url_con);
 
    url_con->active = 1;
    curl_multi_add_handle(curlm, url_con->curl_easy);
+   /* This one can't be stopped, or the download never start. */
    while (curl_multi_perform(curlm, &still_running) == CURLM_CALL_MULTI_PERFORM);
 
    completed_immediately =  _ecore_con_url_process_completed_jobs(url_con);
@@ -473,9 +788,12 @@ _ecore_con_url_perform(Ecore_Con_Url *url_con)
 		  if (flags)
 		    {
 		       FD_SET(fd, &_current_fd_set);
+		       url_con->fd = fd;
+		       url_con->flags = flags;
 		       url_con->fd_handler = ecore_main_fd_handler_add(fd, flags,
 								       _ecore_con_url_fd_handler,
 								       NULL, NULL, NULL);
+		       break;
 		    }
 	       }
 	  }
@@ -484,6 +802,7 @@ _ecore_con_url_perform(Ecore_Con_Url *url_con)
 	     /* Failed to set up an fd_handler */
 	     curl_multi_remove_handle(curlm, url_con->curl_easy);
 	     url_con->active = 0;
+	     url_con->fd = -1;
 	     return 0;
 	  }
      }
@@ -492,21 +811,50 @@ _ecore_con_url_perform(Ecore_Con_Url *url_con)
 }
 
 static int
-_ecore_con_url_fd_handler(void *data __UNUSED__, Ecore_Fd_Handler *fd_handler __UNUSED__)
+_ecore_con_url_idler_handler(void *data __UNUSED__)
 {
-   int still_running;
+   double	start;
+   int		done = 1;
+   int		still_running;
 
-   /* FIXME: Can this run for a long time? Maybe limit how long it can run */
-   while (curl_multi_perform(curlm, &still_running) == CURLM_CALL_MULTI_PERFORM);
+   start = ecore_time_get();
+   while (curl_multi_perform(curlm, &still_running) == CURLM_CALL_MULTI_PERFORM)
+     /* make this not more than a frametime to keep interactivity high */
+     if ((ecore_time_get() - start) > ecore_animator_frametime_get())
+       {
+	  done = 0;
+	  break;
+       }
 
    _ecore_con_url_process_completed_jobs(NULL);
+
+   if (done)
+     {
+	_ecore_con_url_restart_fd_handler();
+	_fd_idler_handler = NULL;
+	return 0;
+     }
+
+   return 1;
+}
+
+static int
+_ecore_con_url_fd_handler(void *data __UNUSED__, Ecore_Fd_Handler *fd_handler __UNUSED__)
+{
+   _ecore_con_url_suspend_fd_handler();
+
+   if (_fd_idler_handler == NULL)
+     _fd_idler_handler = ecore_idler_add(_ecore_con_url_idler_handler, NULL);
+
    return 1;
 }
 
 static int
 _ecore_con_url_process_completed_jobs(Ecore_Con_Url *url_con_to_match)
 {
+   Eina_List *l;
    Ecore_Con_Url *url_con;
+   Ecore_Con_Event_Url_Complete *e;
    CURLMsg *curlmsg;
    int n_remaining;
    int job_matched = 0;
@@ -517,45 +865,41 @@ _ecore_con_url_process_completed_jobs(Ecore_Con_Url *url_con_to_match)
 	if (curlmsg->msg != CURLMSG_DONE) continue;
 
 	/* find the job which is done */
-	ecore_list_first_goto(_url_con_list);
-	while ((url_con = ecore_list_current(_url_con_list)))
+	EINA_LIST_FOREACH(_url_con_list, l, url_con)
 	  {
 	     if (curlmsg->easy_handle == url_con->curl_easy)
 	       {
-		  /* We have found the completed job in our job list */
-		  if (url_con_to_match && (url_con == url_con_to_match)) {
+		  if (url_con_to_match && (url_con == url_con_to_match))
 		       job_matched = 1;
-		  }
-		  if (url_con->fd_handler)
+		  if(url_con->fd != -1)
 		    {
-		       FD_CLR(ecore_main_fd_handler_fd_get(url_con->fd_handler),
-			      &_current_fd_set);
-		       ecore_main_fd_handler_del(url_con->fd_handler);
+		       FD_CLR(url_con->fd, &_current_fd_set);
+		       if (url_con->fd_handler)
+			 ecore_main_fd_handler_del(url_con->fd_handler);
+		       url_con->fd = -1;
 		       url_con->fd_handler = NULL;
 		    }
-		  ecore_list_remove(_url_con_list);
+		  _url_con_list = eina_list_remove(_url_con_list, url_con);
 		  url_con->active = 0;
+		  e = calloc(1, sizeof(Ecore_Con_Event_Url_Complete));
+		  if (e)
 		    {
-		       Ecore_Con_Event_Url_Complete *e;
-		       e = calloc(1, sizeof(Ecore_Con_Event_Url_Complete));
-		       if (e)
-			 {
-			    e->url_con = url_con;
-
-			    e->status = 0;
-			    curl_easy_getinfo(curlmsg->easy_handle, CURLINFO_RESPONSE_CODE, &e->status);
-
-			    _url_complete_push_event(ECORE_CON_EVENT_URL_COMPLETE, e);
-			 }
+		       long status;	/* curl API uses long, not int */
+		       e->url_con = url_con;
+		       e->status = 0;
+		       curl_easy_getinfo(curlmsg->easy_handle, CURLINFO_RESPONSE_CODE, &status);
+		       e->status = status;
+		       _url_complete_push_event(ECORE_CON_EVENT_URL_COMPLETE, e);
 		    }
 		  curl_multi_remove_handle(curlm, url_con->curl_easy);
 		  break;
 	       }
-	     ecore_list_next(_url_con_list);
 	  }
      }
+
    return job_matched;
 }
+
 static void
 _ecore_con_event_url_free(void *data __UNUSED__, void *ev)
 {
