@@ -1,13 +1,19 @@
+/*
+ * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
+ */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
+#include <string.h>
+
 #include "Ecore.h"
+#include "Ecore_Input.h"
+
 #include "ecore_private.h"
 #include "ecore_evas_private.h"
 #include "Ecore_Evas.h"
-#include <string.h>
 
 static int _ecore_evas_init_count = 0;
 static Ecore_Fd_Handler *_ecore_evas_async_events_fd = NULL;
@@ -27,7 +33,7 @@ ecore_evas_engine_type_supported_get(Ecore_Evas_Engine_Type engine)
    switch (engine)
      {
       case ECORE_EVAS_ENGINE_SOFTWARE_BUFFER:
-#ifdef BUILD_ECORE_EVAS_BUFFER
+#ifdef BUILD_ECORE_EVAS_SOFTWARE_BUFFER
 	return 1;
 #else
 	return 0;
@@ -61,6 +67,12 @@ ecore_evas_engine_type_supported_get(Ecore_Evas_Engine_Type engine)
 	return 1;
 #else
 	return 0;
+#endif
+      case ECORE_EVAS_ENGINE_SOFTWARE_GDI:
+#ifdef BUILD_ECORE_EVAS_SOFTWARE_GDI
+	return 1;
+#else
+	return 0;
 #endif
       case ECORE_EVAS_ENGINE_SOFTWARE_DDRAW:
 #ifdef BUILD_ECORE_EVAS_SOFTWARE_DDRAW
@@ -80,8 +92,8 @@ ecore_evas_engine_type_supported_get(Ecore_Evas_Engine_Type engine)
 #else
 	return 0;
 #endif
-     case ECORE_EVAS_ENGINE_SDL:
-#ifdef BUILD_ECORE_EVAS_SDL
+     case ECORE_EVAS_ENGINE_SOFTWARE_SDL:
+#ifdef BUILD_ECORE_EVAS_SOFTWARE_SDL
         return 1;
 #else
         return 0;
@@ -171,7 +183,7 @@ ecore_evas_shutdown(void)
 #ifdef BUILD_ECORE_EVAS_FB
 	while (_ecore_evas_fb_shutdown());
 #endif
-#ifdef BUILD_ECORE_EVAS_BUFFER
+#ifdef BUILD_ECORE_EVAS_SOFTWARE_BUFFER
 	while (_ecore_evas_buffer_shutdown());
 #endif
 #ifdef BUILD_ECORE_EVAS_DIRECTFB
@@ -196,7 +208,7 @@ struct ecore_evas_engine {
 };
 
 /* inline is just to avoid need to ifdef around it */
-static inline void
+static inline const char *
 _ecore_evas_parse_extra_options_str(const char *extra_options, const char *key, char **value)
 {
    int len = strlen(key);
@@ -229,6 +241,7 @@ _ecore_evas_parse_extra_options_str(const char *extra_options, const char *key,
 	     extra_options = NULL;
 	  }
      }
+   return extra_options;
 }
 
 /* inline is just to avoid need to ifdef around it */
@@ -258,6 +271,7 @@ _ecore_evas_parse_extra_options_uint(const char *extra_options, const char *key,
 	else
 	  extra_options = NULL;
      }
+   return extra_options;
 }
 
 /* inline is just to avoid need to ifdef around it */
@@ -333,9 +347,9 @@ _ecore_evas_constructor_software_16_x11(int x, int y, int w, int h, const char *
 }
 #endif
 
-#ifdef BUILD_ECORE_EVAS_SDL
+#ifdef BUILD_ECORE_EVAS_SOFTWARE_SDL
 static Ecore_Evas *
-_ecore_evas_constructor_sdl(int x, int y, int w, int h, const char *extra_options)
+_ecore_evas_constructor_sdl(int x __UNUSED__, int y __UNUSED__, int w, int h, const char *extra_options)
 {
    Ecore_Evas *ee;
    unsigned int fullscreen = 0, hwsurface = 0, noframe = 0, alpha = 0;
@@ -353,7 +367,7 @@ _ecore_evas_constructor_sdl(int x, int y, int w, int h, const char *extra_option
 }
 
 static Ecore_Evas *
-_ecore_evas_constructor_sdl16(int x, int y, int w, int h, const char *extra_options)
+_ecore_evas_constructor_sdl16(int x __UNUSED__, int y __UNUSED__, int w, int h, const char *extra_options)
 {
    Ecore_Evas *ee;
    unsigned int fullscreen = 0, hwsurface = 0, noframe = 0, alpha = 0;
@@ -391,7 +405,7 @@ _ecore_evas_constructor_directfb(int x, int y, int w, int h, const char *extra_o
 
 #ifdef BUILD_ECORE_EVAS_FB
 static Ecore_Evas *
-_ecore_evas_constructor_fb(int x, int y, int w, int h, const char *extra_options)
+_ecore_evas_constructor_fb(int x __UNUSED__, int y __UNUSED__, int w, int h, const char *extra_options)
 {
    Ecore_Evas *ee;
    char *disp_name = NULL;
@@ -407,6 +421,14 @@ _ecore_evas_constructor_fb(int x, int y, int w, int h, const char *extra_options
 }
 #endif
 
+#ifdef BUILD_ECORE_EVAS_SOFTWARE_GDI
+static Ecore_Evas *
+_ecore_evas_constructor_software_gdi(int x, int y, int w, int h, const char *extra_options)
+{
+   return ecore_evas_software_gdi_new(NULL, x, y, w, h);
+}
+#endif
+
 #ifdef BUILD_ECORE_EVAS_SOFTWARE_DDRAW
 static Ecore_Evas *
 _ecore_evas_constructor_software_ddraw(int x, int y, int w, int h, const char *extra_options)
@@ -435,7 +457,7 @@ _ecore_evas_constructor_opengl_glew(int x, int y, int w, int h, const char *extr
 static Ecore_Evas *
 _ecore_evas_constructor_software_16_ddraw(int x, int y, int w, int h, const char *extra_options)
 {
-   return ecore_evas_software_ddraw_new(NULL, x, y, w, h);
+   return ecore_evas_software_16_ddraw_new(NULL, x, y, w, h);
 }
 #endif
 
@@ -457,11 +479,17 @@ _ecore_evas_constructor_software_16_wince_gapi(int x, int y, int w, int h, const
 {
    return ecore_evas_software_wince_gapi_new(NULL, x, y, w, h);
 }
+
+static Ecore_Evas *
+_ecore_evas_constructor_software_16_wince_gdi(int x, int y, int w, int h, const char *extra_options)
+{
+   return ecore_evas_software_wince_gdi_new(NULL, x, y, w, h);
+}
 #endif
 
-#ifdef BUILD_ECORE_EVAS_BUFFER
+#ifdef BUILD_ECORE_EVAS_SOFTWARE_BUFFER
 static Ecore_Evas *
-_ecore_evas_constructor_buffer(int x, int y, int w, int h, const char *extra_options)
+_ecore_evas_constructor_buffer(int x __UNUSED__, int y __UNUSED__, int w, int h, const char *extra_options __UNUSED__)
 {
    return ecore_evas_buffer_new(w, h);
 }
@@ -496,6 +524,9 @@ static const struct ecore_evas_engine _engines[] = {
 #endif
 
   /* windows */
+#ifdef BUILD_ECORE_EVAS_SOFTWARE_GDI
+  {"software_gdi", _ecore_evas_constructor_software_gdi},
+#endif
 #ifdef BUILD_ECORE_EVAS_SOFTWARE_DDRAW
   {"software_ddraw", _ecore_evas_constructor_software_ddraw},
 #endif
@@ -512,16 +543,17 @@ static const struct ecore_evas_engine _engines[] = {
   {"software_16_wince", _ecore_evas_constructor_software_16_wince},
   {"software_16_wince_fb", _ecore_evas_constructor_software_16_wince_fb},
   {"software_16_wince_gapi", _ecore_evas_constructor_software_16_wince_gapi},
+  {"software_16_wince_gdi", _ecore_evas_constructor_software_16_wince_gdi},
 #endif
 
   /* Last chance to have a window */
-#ifdef BUILD_ECORE_EVAS_SDL
+#ifdef BUILD_ECORE_EVAS_SOFTWARE_SDL
   {"sdl", _ecore_evas_constructor_sdl},
   {"software_16_sdl", _ecore_evas_constructor_sdl16},
 #endif
 
   /* independent */
-#ifdef BUILD_ECORE_EVAS_BUFFER
+#ifdef BUILD_ECORE_EVAS_SOFTWARE_BUFFER
   {"buffer", _ecore_evas_constructor_buffer},
 #endif
   {NULL, NULL}
@@ -534,14 +566,14 @@ static const struct ecore_evas_engine _engines[] = {
  * strings are internal and should be considered constants, do not
  * free them, to avoid problems use ecore_evas_engines_free()
  */
-EAPI Evas_List *
+EAPI Eina_List *
 ecore_evas_engines_get(void)
 {
    const struct ecore_evas_engine *itr;
-   Evas_List *lst = NULL;
+   Eina_List *lst = NULL;
 
    for (itr = _engines; itr->name != NULL; itr++)
-     lst = evas_list_append(lst, itr->name);
+     lst = eina_list_append(lst, itr->name);
 
    return lst;
 }
@@ -550,9 +582,9 @@ ecore_evas_engines_get(void)
  * Free list returned by ecore_evas_engines_get()
  */
 EAPI void
-ecore_evas_engines_free(Evas_List *engines)
+ecore_evas_engines_free(Eina_List *engines)
 {
-   evas_list_free(engines);
+   eina_list_free(engines);
 }
 
 static Ecore_Evas *
@@ -608,6 +640,20 @@ ecore_evas_new(const char *engine_name, int x, int y, int w, int h, const char *
    return NULL;
 }
 
+/**
+ * Get the engine name used by this engine.
+ *
+ * should return one of the values in ecore_evas_engines_get(), usually
+ * acceptable by ecore_evas_new().
+ */
+EAPI const char *
+ecore_evas_engine_name_get(const Ecore_Evas *ee)
+{
+   if (!ee)
+     return NULL;
+   return ee->driver;
+}
+
 /**
  * Return the Ecore_Evas for this Evas
  *
@@ -615,7 +661,7 @@ ecore_evas_new(const char *engine_name, int x, int y, int w, int h, const char *
  * @return The Ecore_Evas that holds this Evas
  */
 EAPI Ecore_Evas *
-ecore_evas_ecore_evas_get(Evas *e)
+ecore_evas_ecore_evas_get(const Evas *e)
 {
    return evas_data_attach_get(e);
 }
@@ -653,7 +699,7 @@ ecore_evas_free(Ecore_Evas *ee)
  * @see ecore_evas_data_set
  */
 EAPI void *
-ecore_evas_data_get(Ecore_Evas *ee, const char *key)
+ecore_evas_data_get(const Ecore_Evas *ee, const char *key)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -663,8 +709,9 @@ ecore_evas_data_get(Ecore_Evas *ee, const char *key)
      }
 
    if (!key) return NULL;
+   if (!ee->data) return NULL;
 
-   return evas_hash_find(ee->data, key);
+   return eina_hash_find(ee->data, key);
 }
 
 /**
@@ -694,9 +741,14 @@ ecore_evas_data_set(Ecore_Evas *ee, const char *key, const void *data)
 
    if (!key) return;
 
-   ee->data = evas_hash_del(ee->data, key, NULL);
+   if (ee->data)
+     eina_hash_del(ee->data, key, NULL);
    if (data)
-     ee->data = evas_hash_add(ee->data, key, data);
+     {
+       if (!ee->data)
+	 ee->data = eina_hash_string_superfast_new(NULL);
+       eina_hash_add(ee->data, key, data);
+     }
 }
 
 #define IFC(_ee, _fn)  if (_ee->engine.func->_fn) {_ee->engine.func->_fn
@@ -1010,6 +1062,26 @@ ecore_evas_callback_post_render_set(Ecore_Evas *ee, void (*func) (Ecore_Evas *ee
    ee->func.fn_post_render = func;
 }
 
+/**
+ * Set a callback for Ecore_Evas pre-free event.
+ * @param ee The Ecore_Evas to set callbacks on
+ * @param func The function to call
+ *
+ * A call to this function will set a callback on an Ecore_Evas, causing
+ * @p func to be called just before the instance @p ee is freed.
+ */
+EAPI void
+ecore_evas_callback_pre_free_set(Ecore_Evas *ee, void (*func) (Ecore_Evas *ee))
+{
+   if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
+     {
+	ECORE_MAGIC_FAIL(ee, ECORE_MAGIC_EVAS,
+			 "ecore_evas_callback_pre_free_set");
+	return;
+     }
+   ee->func.fn_pre_free = func;
+}
+
 /**
  * Get an Ecore_Evas's Evas
  * @param ee The Ecore_Evas whose Evas you wish to get
@@ -1018,7 +1090,7 @@ ecore_evas_callback_post_render_set(Ecore_Evas *ee, void (*func) (Ecore_Evas *ee
  * This function returns the Evas contained within @p ee.
  */
 EAPI Evas *
-ecore_evas_get(Ecore_Evas *ee)
+ecore_evas_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1046,6 +1118,7 @@ ecore_evas_move(Ecore_Evas *ee, int x, int y)
 			 "ecore_evas_move");
 	return;
      }
+   if (ee->prop.fullscreen) return;
    IFC(ee, fn_move) (ee, x, y);
    IFE;
 }
@@ -1088,6 +1161,7 @@ ecore_evas_resize(Ecore_Evas *ee, int w, int h)
 			 "ecore_evas_resize");
 	return;
      }
+   if (ee->prop.fullscreen) return;
    if (w < 1) w = 1;
    if (h < 1) h = 1;
    if ((ee->rotation == 90) || (ee->rotation == 270))
@@ -1123,6 +1197,7 @@ ecore_evas_move_resize(Ecore_Evas *ee, int x, int y, int w, int h)
 			 "ecore_evas_move_resize");
 	return;
      }
+   if (ee->prop.fullscreen) return;
    if (w < 1) w = 1;
    if (h < 1) h = 1;
    if ((ee->rotation == 90) || (ee->rotation == 270))
@@ -1155,7 +1230,7 @@ ecore_evas_move_resize(Ecore_Evas *ee, int x, int y, int w, int h)
  *
  */
 EAPI void
-ecore_evas_geometry_get(Ecore_Evas *ee, int *x, int *y, int *w, int *h)
+ecore_evas_geometry_get(const Ecore_Evas *ee, int *x, int *y, int *w, int *h)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1201,6 +1276,9 @@ ecore_evas_rotation_set(Ecore_Evas *ee, int rot)
    while (rot < 0) rot += 360;
    while (rot >= 360) rot -= 360;
    IFC(ee, fn_rotation_set) (ee, rot);
+   /* make sure everything gets redrawn */
+   evas_damage_rectangle_add(ee->evas, 0, 0, ee->w, ee->h);
+   evas_damage_rectangle_add(ee->evas, 0, 0, ee->h, ee->w);
    IFE;
 }
 
@@ -1212,7 +1290,7 @@ ecore_evas_rotation_set(Ecore_Evas *ee, int rot)
  *
  */
 EAPI int
-ecore_evas_rotation_get(Ecore_Evas *ee)
+ecore_evas_rotation_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1256,7 +1334,7 @@ ecore_evas_shaped_set(Ecore_Evas *ee, int shaped)
  * This function returns 1 if @p ee is shaped, and 0 if not.
  */
 EAPI int
-ecore_evas_shaped_get(Ecore_Evas *ee)
+ecore_evas_shaped_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1301,7 +1379,7 @@ ecore_evas_alpha_set(Ecore_Evas *ee, int alpha)
  * it does not.
  */
 EAPI int
-ecore_evas_alpha_get(Ecore_Evas *ee)
+ecore_evas_alpha_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1358,7 +1436,7 @@ ecore_evas_hide(Ecore_Evas *ee)
  * This function queries @p ee and returns 1 if it is visible, and 0 if not.
  */
 EAPI int
-ecore_evas_visibility_get(Ecore_Evas *ee)
+ecore_evas_visibility_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1454,7 +1532,7 @@ ecore_evas_title_set(Ecore_Evas *ee, const char *t)
  * This function returns the title of @p ee.
  */
 EAPI const char *
-ecore_evas_title_get(Ecore_Evas *ee)
+ecore_evas_title_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1496,7 +1574,7 @@ ecore_evas_name_class_set(Ecore_Evas *ee, const char *n, const char *c)
  * @p c.
  */
 EAPI void
-ecore_evas_name_class_get(Ecore_Evas *ee, const char **n, const char **c)
+ecore_evas_name_class_get(const Ecore_Evas *ee, const char **n, const char **c)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1548,7 +1626,7 @@ ecore_evas_size_min_set(Ecore_Evas *ee, int w, int h)
  * This function puts the minimum size of @p ee into @p w and @p h.
  */
 EAPI void
-ecore_evas_size_min_get(Ecore_Evas *ee, int *w, int *h)
+ecore_evas_size_min_get(const Ecore_Evas *ee, int *w, int *h)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1608,7 +1686,7 @@ ecore_evas_size_max_set(Ecore_Evas *ee, int w, int h)
  * This function puts the maximum size of @p ee into @p w and @p h.
  */
 EAPI void
-ecore_evas_size_max_get(Ecore_Evas *ee, int *w, int *h)
+ecore_evas_size_max_get(const Ecore_Evas *ee, int *w, int *h)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1668,7 +1746,7 @@ ecore_evas_size_base_set(Ecore_Evas *ee, int w, int h)
  * This function puts the base size of @p ee into @p w and @p h.
  */
 EAPI void
-ecore_evas_size_base_get(Ecore_Evas *ee, int *w, int *h)
+ecore_evas_size_base_get(const Ecore_Evas *ee, int *w, int *h)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1729,7 +1807,7 @@ ecore_evas_size_step_set(Ecore_Evas *ee, int w, int h)
  * This function puts the step size of @p ee into @p w and @p h.
  */
 EAPI void
-ecore_evas_size_step_get(Ecore_Evas *ee, int *w, int *h)
+ecore_evas_size_step_get(const Ecore_Evas *ee, int *w, int *h)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1796,7 +1874,7 @@ ecore_evas_cursor_set(Ecore_Evas *ee, const char *file, int layer, int hot_x, in
  * @param layer
  * @param hot_x The x coordinate of the cursor's hot spot
  * @param hot_y The y coordinate of the cursor's hot spot
- * 
+ *
  * This function makes the mouse cursor over @p ee be the image specified by
  * @p file. The actual point within the image that the mouse is at is specified
  * by @p hot_x and @p hot_y, which are coordinates with respect to the top left
@@ -1826,7 +1904,7 @@ ecore_evas_object_cursor_set(Ecore_Evas *ee, Evas_Object *obj, int layer, int ho
  * This function queries information about an Ecore_Evas' cursor.
  */
 EAPI void
-ecore_evas_cursor_get(Ecore_Evas *ee, Evas_Object **obj, int *layer, int *hot_x, int *hot_y)
+ecore_evas_cursor_get(const Ecore_Evas *ee, Evas_Object **obj, int *layer, int *hot_x, int *hot_y)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1867,7 +1945,7 @@ ecore_evas_layer_set(Ecore_Evas *ee, int layer)
  *
  */
 EAPI int
-ecore_evas_layer_get(Ecore_Evas *ee)
+ecore_evas_layer_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1905,7 +1983,7 @@ ecore_evas_focus_set(Ecore_Evas *ee, int on)
  *
  */
 EAPI int
-ecore_evas_focus_get(Ecore_Evas *ee)
+ecore_evas_focus_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1944,7 +2022,7 @@ ecore_evas_iconified_set(Ecore_Evas *ee, int on)
  *
  */
 EAPI int
-ecore_evas_iconified_get(Ecore_Evas *ee)
+ecore_evas_iconified_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -1983,7 +2061,7 @@ ecore_evas_borderless_set(Ecore_Evas *ee, int on)
  *
  */
 EAPI int
-ecore_evas_borderless_get(Ecore_Evas *ee)
+ecore_evas_borderless_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -2022,7 +2100,7 @@ ecore_evas_override_set(Ecore_Evas *ee, int on)
  *
  */
 EAPI int
-ecore_evas_override_get(Ecore_Evas *ee)
+ecore_evas_override_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -2061,7 +2139,7 @@ ecore_evas_maximized_set(Ecore_Evas *ee, int on)
  *
  */
 EAPI int
-ecore_evas_maximized_get(Ecore_Evas *ee)
+ecore_evas_maximized_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -2100,7 +2178,7 @@ ecore_evas_fullscreen_set(Ecore_Evas *ee, int on)
  *
  */
 EAPI int
-ecore_evas_fullscreen_get(Ecore_Evas *ee)
+ecore_evas_fullscreen_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -2140,7 +2218,7 @@ ecore_evas_avoid_damage_set(Ecore_Evas *ee, Ecore_Evas_Avoid_Damage_Type on)
  *
  */
 EAPI Ecore_Evas_Avoid_Damage_Type
-ecore_evas_avoid_damage_get(Ecore_Evas *ee)
+ecore_evas_avoid_damage_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
      {
@@ -2178,7 +2256,7 @@ ecore_evas_withdrawn_set(Ecore_Evas *ee, int withdrawn)
  *
  */
 EAPI int
-ecore_evas_withdrawn_get(Ecore_Evas *ee)
+ecore_evas_withdrawn_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
    {
@@ -2218,7 +2296,7 @@ ecore_evas_sticky_set(Ecore_Evas *ee, int sticky)
  *
  */
 EAPI int
-ecore_evas_sticky_get(Ecore_Evas *ee)
+ecore_evas_sticky_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
    {
@@ -2258,7 +2336,7 @@ ecore_evas_ignore_events_set(Ecore_Evas *ee, int ignore)
  *
  */
 EAPI int
-ecore_evas_ignore_events_get(Ecore_Evas *ee)
+ecore_evas_ignore_events_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
    {
@@ -2269,18 +2347,17 @@ ecore_evas_ignore_events_get(Ecore_Evas *ee)
       return ee->ignore_events ? 1 : 0;
 }
 
-EAPI void *
-ecore_evas_window_get(Ecore_Evas *ee)
+EAPI Ecore_Window
+ecore_evas_window_get(const Ecore_Evas *ee)
 {
    if (!ECORE_MAGIC_CHECK(ee, ECORE_MAGIC_EVAS))
    {
       ECORE_MAGIC_FAIL(ee, ECORE_MAGIC_EVAS,
          "ecore_evas_window_get");
-      return NULL;
+      return 0;
    }
 
-   if (ee->engine.func->fn_window_get) return ee->engine.func->fn_window_get(ee);
-   return NULL;
+   return ee->prop.window;
 }
 
 /* fps debug calls - for debugging how much time your app actually spends */
@@ -2342,31 +2419,43 @@ _ecore_evas_fps_debug_shutdown(void)
 void
 _ecore_evas_fps_debug_rendertime_add(double t)
 {
-   if ((_ecore_evas_fps_debug_fd >= 0) &&
-       (_ecore_evas_fps_rendertime_mmap))
-     {
-	unsigned int tm;
+   static double rtime = 0.0;
+   static double rlapse = 0.0;
+   static int frames = 0;
+   static int flapse = 0;
+   double tim;
 
-	tm = (unsigned int)(t * 1000000.0);
-	/* i know its not 100% theoretically guaranteed, but i'd say a write */
-	/* of an int could be considered atomic for all practical purposes */
-	/* oh and since this is cumulative, 1 second = 1,000,000 ticks, so */
-	/* this can run for about 4294 seconds becore looping. if you are */
-	/* doing performance testing in one run for over an hour... well */
-	/* time to restart or handle a loop condition :) */
-	*(_ecore_evas_fps_rendertime_mmap) += tm;
+   tim = ecore_time_get();
+   rtime += t;
+   frames++;
+   if (rlapse == 0.0)
+     {
+        rlapse = tim;
+        flapse = frames;
+     }
+   else if ((tim - rlapse) >= 0.5)
+     {
+        printf("FRAME: %i, FPS: %3.1f, RTIME %3.0f%%\n",
+               frames,
+               (frames - flapse) / (tim - rlapse),
+               (100.0 * rtime) / (tim - rlapse)
+               );
+        rlapse = tim;
+        flapse = frames;
+        rtime = 0.0;
      }
 }
 
 void
 _ecore_evas_free(Ecore_Evas *ee)
 {
+   if (ee->func.fn_pre_free) ee->func.fn_pre_free(ee);
    ECORE_MAGIC_SET(ee, ECORE_MAGIC_NONE);
    while (ee->sub_ecore_evas)
      {
 	_ecore_evas_free(ee->sub_ecore_evas->data);
      }
-   if (ee->data) evas_hash_free(ee->data);
+   if (ee->data) eina_hash_free(ee->data);
    if (ee->name) free(ee->name);
    if (ee->prop.title) free(ee->prop.title);
    if (ee->prop.name) free(ee->prop.name);
@@ -2399,7 +2488,7 @@ _ecore_evas_cb_idle_flush(void *data)
 }
 
 static int
-_ecore_evas_async_events_fd_handler(void *data, Ecore_Fd_Handler *fd_handler)
+_ecore_evas_async_events_fd_handler(void *data __UNUSED__, Ecore_Fd_Handler *fd_handler __UNUSED__)
 {
    evas_async_events_process();
 
@@ -2415,3 +2504,39 @@ _ecore_evas_idle_timeout_update(Ecore_Evas *ee)
 						 _ecore_evas_cb_idle_flush,
 						 ee);
 }
+
+void
+_ecore_evas_mouse_move_process(Ecore_Evas *ee, int x, int y, unsigned int timestamp)
+{
+   ee->mouse.x = x;
+   ee->mouse.y = y;
+   if (ee->prop.cursor.object)
+     {
+	evas_object_show(ee->prop.cursor.object);
+	if (ee->rotation == 0)
+	  evas_object_move(ee->prop.cursor.object,
+			   x - ee->prop.cursor.hot.x,
+			   y - ee->prop.cursor.hot.y);
+	else if (ee->rotation == 90)
+	  evas_object_move(ee->prop.cursor.object,
+			   ee->h - y - 1 - ee->prop.cursor.hot.x,
+			   x - ee->prop.cursor.hot.y);
+	else if (ee->rotation == 180)
+	  evas_object_move(ee->prop.cursor.object,
+			   ee->w - x - 1 - ee->prop.cursor.hot.x,
+			   ee->h - y - 1 - ee->prop.cursor.hot.y);
+	else if (ee->rotation == 270)
+	  evas_object_move(ee->prop.cursor.object,
+			   y - ee->prop.cursor.hot.x,
+			   ee->w - x - 1 - ee->prop.cursor.hot.y);
+     }
+   if (ee->rotation == 0)
+     evas_event_feed_mouse_move(ee->evas, x, y, timestamp, NULL);
+   else if (ee->rotation == 90)
+     evas_event_feed_mouse_move(ee->evas, ee->h - y - 1, x, timestamp, NULL);
+   else if (ee->rotation == 180)
+     evas_event_feed_mouse_move(ee->evas, ee->w - x - 1, ee->h - y - 1, timestamp, NULL);
+   else if (ee->rotation == 270)
+     evas_event_feed_mouse_move(ee->evas, y, ee->w - x - 1, timestamp, NULL);
+}
+
