@@ -6,39 +6,41 @@
 # include <config.h>
 #endif
 
+#include <stdio.h>
+#include <string.h>
+
 #include "ecore_file_private.h"
 
 static int init = 0;
-static Ecore_List *__ecore_file_path_bin = NULL;
+static Eina_List *__ecore_file_path_bin = NULL;
 
-static Ecore_List *_ecore_file_path_from_env(const char *env);
+static Eina_List *_ecore_file_path_from_env(const char *env);
 
 int
 ecore_file_path_init(void)
 {
    if (++init != 1) return init;
    __ecore_file_path_bin = _ecore_file_path_from_env("PATH");
-   ecore_list_free_cb_set(__ecore_file_path_bin, free);
    return init;
 }
 
 int
 ecore_file_path_shutdown(void)
 {
+   char *dir;
+
    if (--init != 0) return init;
-   ecore_list_destroy(__ecore_file_path_bin);
-   __ecore_file_path_bin = NULL;
+   EINA_LIST_FREE(__ecore_file_path_bin, dir)
+     free(dir);
    return init;
 }
 
-Ecore_List *
+Eina_List *
 _ecore_file_path_from_env(const char *env)
 {
-   Ecore_List *path;
+   Eina_List *path = NULL;
    char *env_path, *p, *last;
 
-   path = ecore_list_new();
-
    env_path = getenv(env);
    if (!env_path)
      return path;
@@ -53,12 +55,12 @@ _ecore_file_path_from_env(const char *env)
         if (!*p)
           {
 	     if (!ecore_file_path_dir_exists(last))
-	       ecore_list_append(path, strdup(last));
+	       path = eina_list_append(path, strdup(last));
              last = p + 1;
           }
      }
    if (p > last)
-     ecore_list_append(path, strdup(last));
+     path = eina_list_append(path, strdup(last));
 
    free(env_path);
    return path;
@@ -72,14 +74,16 @@ _ecore_file_path_from_env(const char *env)
 EAPI int
 ecore_file_path_dir_exists(const char *in_dir)
 {
+   Eina_List *l;
    char *dir;
 
    if (!__ecore_file_path_bin) return 0;
-   ecore_list_first_goto(__ecore_file_path_bin);
-   while ((dir = ecore_list_next(__ecore_file_path_bin)) != NULL)
+   EINA_LIST_FOREACH(__ecore_file_path_bin, l, dir)
      {
-	if (!strcmp(dir, in_dir)) return 1;
+	if (strcmp(dir, in_dir))
+	  return 1;
      }
+
    return 0;
 }
 
@@ -93,50 +97,47 @@ ecore_file_path_dir_exists(const char *in_dir)
 EAPI int
 ecore_file_app_installed(const char *exe)
 {
+   Eina_List *l;
    char *dir;
    char  buf[PATH_MAX];
 
    if (!exe) return 0;
    if (ecore_file_can_exec(exe)) return 1;
 
-   ecore_list_first_goto(__ecore_file_path_bin);
-   while ((dir = ecore_list_next(__ecore_file_path_bin)) != NULL)
+   EINA_LIST_FOREACH(__ecore_file_path_bin, l, dir)
      {
 	snprintf(buf, sizeof(buf), "%s/%s", dir, exe);
-	if (ecore_file_can_exec(buf)) return 1;
+	if (ecore_file_can_exec(buf))
+	  return 1;
      }
+
    return 0;
 }
 
 /**
  * Get a list of all the applications installed on the system
- * @return An Ecore_List containing all the executable files in the system
+ * @return An Eina_List containing all the executable files in the system
  */
-EAPI Ecore_List *
+EAPI Eina_List *
 ecore_file_app_list(void)
 {
-   Ecore_List *list, *files;
+   Eina_List *list = NULL;
+   Eina_List *files;
+   Eina_List *l;
    char  buf[PATH_MAX], *dir, *exe;
    
-   list = ecore_list_new();
-   if (!list) return NULL;
-   ecore_list_free_cb_set(list, free);
-   ecore_list_first_goto(__ecore_file_path_bin);
-   while ((dir = ecore_list_next(__ecore_file_path_bin)) != NULL)
+   EINA_LIST_FOREACH(__ecore_file_path_bin, l, dir)
      {
 	files = ecore_file_ls(dir);
-	if (files)
-	  {
-	     ecore_list_first_goto(files);
-	     while ((exe = ecore_list_next(files)) != NULL)
+	EINA_LIST_FREE(files, exe)
 	       {
 		  snprintf(buf, sizeof(buf), "%s/%s", dir, exe);
 		  if ((ecore_file_can_exec(buf)) &&
 		      (!ecore_file_is_dir(buf)))
-		    ecore_list_append(list, strdup(buf));
-	       }
-	     ecore_list_destroy(files);
+	       list = eina_list_append(list, strdup(buf));
+	     free(exe);
 	  }
      }
+
    return list;
 }
