@@ -33,7 +33,12 @@
 #endif /* ! _WIN32 */
 
 
-typedef void Ecore_Win32_Window;
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+
+typedef struct _Ecore_Win32_Window Ecore_Win32_Window;
 typedef void Ecore_Win32_Cursor;
 
 /* Window state */
@@ -116,14 +121,8 @@ typedef enum
 
 /* Events */
 
-typedef struct _Ecore_Win32_Event_Key_Down              Ecore_Win32_Event_Key_Down;
-typedef struct _Ecore_Win32_Event_Key_Up                Ecore_Win32_Event_Key_Up;
-typedef struct _Ecore_Win32_Event_Mouse_Button_Down     Ecore_Win32_Event_Mouse_Button_Down;
-typedef struct _Ecore_Win32_Event_Mouse_Button_Up       Ecore_Win32_Event_Mouse_Button_Up;
-typedef struct _Ecore_Win32_Event_Mouse_Move            Ecore_Win32_Event_Mouse_Move;
 typedef struct _Ecore_Win32_Event_Mouse_In              Ecore_Win32_Event_Mouse_In;
 typedef struct _Ecore_Win32_Event_Mouse_Out             Ecore_Win32_Event_Mouse_Out;
-typedef struct _Ecore_Win32_Event_Mouse_Wheel           Ecore_Win32_Event_Mouse_Wheel;
 typedef struct _Ecore_Win32_Event_Window_Focus_In       Ecore_Win32_Event_Window_Focus_In;
 typedef struct _Ecore_Win32_Event_Window_Focus_Out      Ecore_Win32_Event_Window_Focus_Out;
 typedef struct _Ecore_Win32_Event_Window_Damage         Ecore_Win32_Event_Window_Damage;
@@ -135,54 +134,6 @@ typedef struct _Ecore_Win32_Event_Window_Configure      Ecore_Win32_Event_Window
 typedef struct _Ecore_Win32_Event_Window_Resize         Ecore_Win32_Event_Window_Resize;
 typedef struct _Ecore_Win32_Event_Window_Delete_Request Ecore_Win32_Event_Window_Delete_Request;
 
-struct _Ecore_Win32_Event_Key_Down
-{
-   Ecore_Win32_Window *window;
-   char               *keyname;
-   char               *keysymbol;
-   char               *keycompose;
-   double              time;
-};
-
-struct _Ecore_Win32_Event_Key_Up
-{
-   Ecore_Win32_Window *window;
-   char               *keyname;
-   char               *keysymbol;
-   char               *keycompose;
-   double              time;
-};
-
-struct _Ecore_Win32_Event_Mouse_Button_Down
-{
-   Ecore_Win32_Window *window;
-   int                 button;
-   int                 x;
-   int                 y;
-   double              time;
-   unsigned int        double_click : 1;
-   unsigned int        triple_click : 1;
-};
-
-struct _Ecore_Win32_Event_Mouse_Button_Up
-{
-   Ecore_Win32_Window *window;
-   int                 button;
-   int                 x;
-   int                 y;
-   double              time;
-   unsigned int        double_click : 1;
-   unsigned int        triple_click : 1;
-};
-
-struct _Ecore_Win32_Event_Mouse_Move
-{
-   Ecore_Win32_Window *window;
-   int                 x;
-   int                 y;
-   double              time;
-};
-
 struct _Ecore_Win32_Event_Mouse_In
 {
    Ecore_Win32_Window *window;
@@ -199,17 +150,6 @@ struct _Ecore_Win32_Event_Mouse_Out
    double              time;
 };
 
-struct _Ecore_Win32_Event_Mouse_Wheel
-{
-   Ecore_Win32_Window *window;
-   int                 direction; /* 0 = default up/down wheel FIXME: more wheel types */
-   int                 z; /* ...,-2,-1 = down, 1,2,... = up */
-   int                 modifiers;
-   int                 x;
-   int                 y;
-   double              time;
-};
-
 struct _Ecore_Win32_Event_Window_Focus_In
 {
    Ecore_Win32_Window *window;
@@ -281,14 +221,16 @@ struct _Ecore_Win32_Event_Window_Delete_Request
    double              time;
 };
 
-EAPI extern int ECORE_WIN32_EVENT_KEY_DOWN;
-EAPI extern int ECORE_WIN32_EVENT_KEY_UP;
-EAPI extern int ECORE_WIN32_EVENT_MOUSE_BUTTON_DOWN;
-EAPI extern int ECORE_WIN32_EVENT_MOUSE_BUTTON_UP;
-EAPI extern int ECORE_WIN32_EVENT_MOUSE_MOVE;
+#define ECORE_WIN32_DND_EVENT_DRAG_ENTER     1
+#define ECORE_WIN32_DND_EVENT_DRAG_OVER      2
+#define ECORE_WIN32_DND_EVENT_DRAG_LEAVE     3
+#define ECORE_WIN32_DND_EVENT_DROP           4
+
+
+typedef int (*Ecore_Win32_Dnd_DropTarget_Callback)(void *window, int event, int pt_x, int pt_y, void *data, int size);
+
 EAPI extern int ECORE_WIN32_EVENT_MOUSE_IN;
 EAPI extern int ECORE_WIN32_EVENT_MOUSE_OUT;
-EAPI extern int ECORE_WIN32_EVENT_MOUSE_WHEEL;
 EAPI extern int ECORE_WIN32_EVENT_WINDOW_FOCUS_IN;
 EAPI extern int ECORE_WIN32_EVENT_WINDOW_FOCUS_OUT;
 EAPI extern int ECORE_WIN32_EVENT_WINDOW_DAMAGE;
@@ -322,7 +264,7 @@ EAPI Ecore_Win32_Window *ecore_win32_window_override_new(Ecore_Win32_Window *par
                                                          int                 width,
                                                          int                 height);
 
-EAPI void ecore_win32_window_del(Ecore_Win32_Window *window);
+EAPI void ecore_win32_window_free(Ecore_Win32_Window *window);
 
 EAPI void *ecore_win32_window_hwnd_get(Ecore_Win32_Window *window);
 
@@ -404,6 +346,11 @@ EAPI void ecore_win32_window_borderless_set(Ecore_Win32_Window *window,
 EAPI void ecore_win32_window_fullscreen_set(Ecore_Win32_Window *window,
                                             int                 on);
 
+EAPI void ecore_win32_window_shape_set(Ecore_Win32_Window *window,
+                                       unsigned short      width,
+                                       unsigned short      height,
+                                       unsigned char      *mask);
+
 EAPI void ecore_win32_window_cursor_set(Ecore_Win32_Window *window,
                                         Ecore_Win32_Cursor *cursor);
 
@@ -434,4 +381,20 @@ EAPI Ecore_Win32_Cursor *ecore_win32_cursor_shape_get(Ecore_Win32_Cursor_Shape s
 EAPI int                 ecore_win32_cursor_size_get(void);
 
 
+
+/* Drag and drop */
+EAPI int ecore_win32_dnd_init();
+EAPI int ecore_win32_dnd_shutdown();
+EAPI int ecore_win32_dnd_begin(const char *data,
+                               int         size);
+EAPI int ecore_win32_dnd_register_drop_target(Ecore_Win32_Window                 *window,
+                                              Ecore_Win32_Dnd_DropTarget_Callback callback);
+EAPI void ecore_win32_dnd_unregister_drop_target(Ecore_Win32_Window *window);
+
+
+#ifdef __cplusplus
+}
+#endif
+
+
 #endif /* __ECORE_WIN32_H__ */
