@@ -1,10 +1,17 @@
 /*
  * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
  */
+
 /*
  * _NET_WM... aka Extended Window Manager Hint (EWMH) functions.
  */
-#include "config.h"
+
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#include <string.h>
+
 #include "Ecore.h"
 #include "Ecore_Data.h"
 #include "ecore_x_private.h"
@@ -56,21 +63,19 @@ static void  _ecore_x_netwm_startup_info_free(void *data);
  * Local variables
  */
 
-static Ecore_Hash *startup_info = NULL;
+static Eina_Hash *startup_info = NULL;
 
 EAPI void
 ecore_x_netwm_init(void)
 {
-   startup_info = ecore_hash_new(ecore_direct_hash, ecore_direct_compare);
-   if (startup_info)
-     ecore_hash_free_value_cb_set(startup_info, _ecore_x_netwm_startup_info_free);
+   startup_info = eina_hash_string_superfast_new(_ecore_x_netwm_startup_info_free);
 }
 
 EAPI void
 ecore_x_netwm_shutdown(void)
 {
    if (startup_info)
-     ecore_hash_destroy(startup_info);
+     eina_hash_free(startup_info);
    startup_info = NULL;
 }
 
@@ -895,6 +900,10 @@ _ecore_x_netwm_action_atom_get(Ecore_X_Action action)
 	 return ECORE_X_ATOM_NET_WM_ACTION_CHANGE_DESKTOP;
       case ECORE_X_ACTION_CLOSE:
 	 return ECORE_X_ATOM_NET_WM_ACTION_CLOSE;
+      case ECORE_X_ACTION_ABOVE:
+	 return ECORE_X_ATOM_NET_WM_ACTION_ABOVE;
+      case ECORE_X_ACTION_BELOW:
+	 return ECORE_X_ATOM_NET_WM_ACTION_BELOW;
       default:
 	 return 0;
      }
@@ -1140,11 +1149,13 @@ _ecore_x_netwm_startup_info_begin(Ecore_X_Window win, char *data)
 {
 #if 0
    Ecore_X_Startup_Info *info;
+   unsigned char	*exists = 0;
 
    if (!startup_info) return 0;
-   info = ecore_hash_get(startup_info, (void *)win);
+   info = eina_hash_find(startup_info, (void *)win);
    if (info)
      {
+        exists = 1;
 	printf("Already got info for win: 0x%x\n", win);
 	_ecore_x_netwm_startup_info_free(info);
      }
@@ -1162,7 +1173,10 @@ _ecore_x_netwm_startup_info_begin(Ecore_X_Window win, char *data)
    memcpy(info->buffer, data, 20);
    info->length += 20;
    info->buffer[info->length] = 0;
-   ecore_hash_set(startup_info, (void *)info->win, info);
+   if (exists)
+     eina_hash_modify(startup_info, (void *)info->win, info);
+   else
+     eina_hash_add(startup_info, (void *)info->win, info);
    if (strlen(info->buffer) != 20)
      {
 	/* We have a '\0' in there, the message is done */
@@ -1183,7 +1197,7 @@ _ecore_x_netwm_startup_info(Ecore_X_Window win, char *data)
    char *p;
 
    if (!startup_info) return 0;
-   info = ecore_hash_get(startup_info, (void *)win);
+   info = eina_hash_find(startup_info, (void *)win);
    if (!info) return 0;
    if ((info->length + 20) > info->buffer_size)
      {
@@ -1191,7 +1205,7 @@ _ecore_x_netwm_startup_info(Ecore_X_Window win, char *data)
 	info->buffer = realloc(info->buffer, info->buffer_size * sizeof(char));
 	if (!info->buffer)
 	  {
-	     ecore_hash_remove(startup_info, (void *)info->win);
+	     eina_hash_del(startup_info, (void *)info->win);
 	     _ecore_x_netwm_startup_info_free(info);
 	     return 0;
 	  }
@@ -1269,7 +1283,7 @@ _ecore_x_netwm_startup_info_process(Ecore_X_Startup_Info *info)
    p = strchr(info->buffer, ':');
    if (!p)
      {
-	ecore_hash_remove(startup_info, (void *)info->win);
+	eina_hash_del(startup_info, (void *)info->win);
 	_ecore_x_netwm_startup_info_free(info);
 	return 0;
      }
@@ -1290,7 +1304,7 @@ _ecore_x_netwm_startup_info_process(Ecore_X_Startup_Info *info)
      event = ECORE_X_EVENT_STARTUP_SEQUENCE_REMOVE;
    else
      {
-	ecore_hash_remove(startup_info, (void *)info->win);
+	eina_hash_del(startup_info, (void *)info->win);
 	_ecore_x_netwm_startup_info_free(info);
 	return 0;
      }
@@ -1299,7 +1313,7 @@ _ecore_x_netwm_startup_info_process(Ecore_X_Startup_Info *info)
 
    if (!_ecore_x_netwm_startup_info_parse(info, p))
      {
-	ecore_hash_remove(startup_info, (void *)info->win);
+	eina_hash_del(startup_info, (void *)info->win);
 	_ecore_x_netwm_startup_info_free(info);
 	return 0;
      }
@@ -1309,7 +1323,7 @@ _ecore_x_netwm_startup_info_process(Ecore_X_Startup_Info *info)
 	e = calloc(1, sizeof(Ecore_X_Event_Startup_Sequence));
 	if (!e)
 	  {
-	     ecore_hash_remove(startup_info, (void *)info->win);
+	     eina_hash_del(startup_info, (void *)info->win);
 	     _ecore_x_netwm_startup_info_free(info);
 	     return 0;
 	  }
@@ -1319,7 +1333,7 @@ _ecore_x_netwm_startup_info_process(Ecore_X_Startup_Info *info)
 
    if (event == ECORE_X_EVENT_STARTUP_SEQUENCE_REMOVE)
      {
-	ecore_hash_remove(startup_info, (void *)info->win);
+	eina_hash_del(startup_info, (void *)info->win);
 	_ecore_x_netwm_startup_info_free(info);
      }
    else
